@@ -1,4 +1,4 @@
-// show the difference between calling a V8 binding C++ function
+// Show the difference between calling a V8 binding C++ function
 // relative to a comparable N-API C++ function,
 // in various types/numbers of arguments.
 // Reports n of calls per second.
@@ -89,7 +89,7 @@ function main({ n, engine, type }) {
     const args = generateArgs(type);
 
     bench.start();
-    for (var i = 0; i < n; i++) {
+    for (let i = 0; i < n; i++) {
       fn.apply(null, args);
     }
     bench.end(n);
