@@ -9,7 +9,6 @@
 #include "src/compiler/js-heap-broker.h"
 #include "src/compiler/js-operator.h"
 #include "src/compiler/linkage.h"
-#include "src/compiler/node-matchers.h"
 #include "src/compiler/node-properties.h"
 #include "src/objects/contexts-inl.h"
 
@@ -40,7 +39,7 @@ Reduction JSContextSpecialization::ReduceParameter(Node* node) {
     // Constant-fold the function parameter {node}.
     Handle<JSFunction> function;
     if (closure().ToHandle(&function)) {
-      Node* value = jsgraph()->Constant(MakeRef(broker_, function));
+      Node* value = jsgraph()->Constant(MakeRef(broker_, function), broker());
       return Replace(value);
     }
   }
@@ -98,9 +97,9 @@ bool IsContextParameter(Node* node) {
 // context (which we want to read from or store to), try to return a
 // specialization context.  If successful, update {distance} to whatever
 // distance remains from the specialization context.
-base::Optional<ContextRef> GetSpecializationContext(
-    JSHeapBroker* broker, Node* node, size_t* distance,
-    Maybe<OuterContext> maybe_outer) {
+OptionalContextRef GetSpecializationContext(JSHeapBroker* broker, Node* node,
+                                            size_t* distance,
+                                            Maybe<OuterContext> maybe_outer) {
   switch (node->opcode()) {
     case IrOpcode::kHeapConstant: {
       // TODO(jgruber,chromium:1209798): Using kAssumeMemoryFence works around
@@ -128,7 +127,7 @@ base::Optional<ContextRef> GetSpecializationContext(
     default:
       break;
   }
-  return base::Optional<ContextRef>();
+  return OptionalContextRef();
 }
 
 }  // anonymous namespace
@@ -142,7 +141,7 @@ Reduction JSContextSpecialization::ReduceJSLoadContext(Node* node) {
   // First walk up the context chain in the graph as far as possible.
   Node* context = NodeProperties::GetOuterContext(node, &depth);
 
-  base::Optional<ContextRef> maybe_concrete =
+  OptionalContextRef maybe_concrete =
       GetSpecializationContext(broker(), context, &depth, outer());
   if (!maybe_concrete.has_value()) {
     // We do not have a concrete context object, so we can only partially reduce
@@ -152,27 +151,30 @@ Reduction JSContextSpecialization::ReduceJSLoadContext(Node* node) {
 
   // Now walk up the concrete context chain for the remaining depth.
   ContextRef concrete = maybe_concrete.value();
-  concrete = concrete.previous(&depth);
+  concrete = concrete.previous(broker(), &depth);
   if (depth > 0) {
     TRACE_BROKER_MISSING(broker(), "previous value for context " << concrete);
-    return SimplifyJSLoadContext(node, jsgraph()->Constant(concrete), depth);
+    return SimplifyJSLoadContext(node, jsgraph()->Constant(concrete, broker()),
+                                 depth);
   }
 
   if (!access.immutable()) {
     // We found the requested context object but since the context slot is
     // mutable we can only partially reduce the load.
-    return SimplifyJSLoadContext(node, jsgraph()->Constant(concrete), depth);
+    return SimplifyJSLoadContext(node, jsgraph()->Constant(concrete, broker()),
+                                 depth);
   }
 
   // This will hold the final value, if we can figure it out.
-  base::Optional<ObjectRef> maybe_value;
-  maybe_value = concrete.get(static_cast<int>(access.index()));
+  OptionalObjectRef maybe_value;
+  maybe_value = concrete.get(broker(), static_cast<int>(access.index()));
 
   if (!maybe_value.has_value()) {
     TRACE_BROKER_MISSING(broker(), "slot value " << access.index()
                                                  << " for context "
                                                  << concrete);
-    return SimplifyJSLoadContext(node, jsgraph()->Constant(concrete), depth);
+    return SimplifyJSLoadContext(node, jsgraph()->Constant(concrete, broker()),
+                                 depth);
   }
 
   if (!maybe_value->IsSmi()) {
@@ -181,15 +183,17 @@ Reduction JSContextSpecialization::ReduceJSLoadContext(Node* node) {
     // We must be conservative and check if the value in the slot is currently
     // the hole or undefined. Only if it is neither of these, can we be sure
     // that it won't change anymore.
-    OddballType oddball_type = maybe_value->AsHeapObject().map().oddball_type();
+    OddballType oddball_type =
+        maybe_value->AsHeapObject().map(broker()).oddball_type(broker());
     if (oddball_type == OddballType::kUndefined ||
         oddball_type == OddballType::kHole) {
-      return SimplifyJSLoadContext(node, jsgraph()->Constant(concrete), depth);
+      return SimplifyJSLoadContext(
+          node, jsgraph()->Constant(concrete, broker()), depth);
     }
   }
 
   // Success. The context load can be replaced with the constant.
-  Node* constant = jsgraph_->Constant(*maybe_value);
+  Node* constant = jsgraph_->Constant(*maybe_value, broker());
   ReplaceWithValue(node, constant);
   return Replace(constant);
 }
@@ -205,7 +209,7 @@ Reduction JSContextSpecialization::ReduceJSStoreContext(Node* node) {
   // or hit a node that does not have a CreateXYZContext operator.
   Node* context = NodeProperties::GetOuterContext(node, &depth);
 
-  base::Optional<ContextRef> maybe_concrete =
+  OptionalContextRef maybe_concrete =
       GetSpecializationContext(broker(), context, &depth, outer());
   if (!maybe_concrete.has_value()) {
     // We do not have a concrete context object, so we can only partially reduce
@@ -215,24 +219,26 @@ Reduction JSContextSpecialization::ReduceJSStoreContext(Node* node) {
 
   // Now walk up the concrete context chain for the remaining depth.
   ContextRef concrete = maybe_concrete.value();
-  concrete = concrete.previous(&depth);
+  concrete = concrete.previous(broker(), &depth);
   if (depth > 0) {
     TRACE_BROKER_MISSING(broker(), "previous value for context " << concrete);
-    return SimplifyJSStoreContext(node, jsgraph()->Constant(concrete), depth);
+    return SimplifyJSStoreContext(node, jsgraph()->Constant(concrete, broker()),
+                                  depth);
   }
 
-  return SimplifyJSStoreContext(node, jsgraph()->Constant(concrete), depth);
+  return SimplifyJSStoreContext(node, jsgraph()->Constant(concrete, broker()),
+                                depth);
 }
 
-base::Optional<ContextRef> GetModuleContext(JSHeapBroker* broker, Node* node,
-                                            Maybe<OuterContext> maybe_context) {
+OptionalContextRef GetModuleContext(JSHeapBroker* broker, Node* node,
+                                    Maybe<OuterContext> maybe_context) {
   size_t depth = std::numeric_limits<size_t>::max();
   Node* context = NodeProperties::GetOuterContext(node, &depth);
 
-  auto find_context = [](ContextRef c) {
-    while (c.map().instance_type() != MODULE_CONTEXT_TYPE) {
+  auto find_context = [broker](ContextRef c) {
+    while (c.map(broker).instance_type() != MODULE_CONTEXT_TYPE) {
       size_t depth = 1;
-      c = c.previous(&depth);
+      c = c.previous(broker, &depth);
       CHECK_EQ(depth, 0);
     }
     return c;
@@ -266,28 +272,27 @@ base::Optional<ContextRef> GetModuleContext(JSHeapBroker* broker, Node* node,
       break;
   }
 
-  return base::Optional<ContextRef>();
+  return OptionalContextRef();
 }
 
 Reduction JSContextSpecialization::ReduceJSGetImportMeta(Node* node) {
-  base::Optional<ContextRef> maybe_context =
-      GetModuleContext(broker(), node, outer());
+  OptionalContextRef maybe_context = GetModuleContext(broker(), node, outer());
   if (!maybe_context.has_value()) return NoChange();
 
   ContextRef context = maybe_context.value();
-  base::Optional<ObjectRef> module = context.get(Context::EXTENSION_INDEX);
+  OptionalObjectRef module = context.get(broker(), Context::EXTENSION_INDEX);
   if (!module.has_value()) return NoChange();
-  base::Optional<ObjectRef> import_meta =
-      module->AsSourceTextModule().import_meta();
+  OptionalObjectRef import_meta =
+      module->AsSourceTextModule().import_meta(broker());
   if (!import_meta.has_value()) return NoChange();
   if (!import_meta->IsJSObject()) {
-    DCHECK(import_meta->IsTheHole());
+    DCHECK(import_meta->IsTheHole(broker()));
     // The import.meta object has not yet been created. Let JSGenericLowering
     // replace the operator with a runtime call.
     return NoChange();
   }
 
-  Node* import_meta_const = jsgraph()->Constant(*import_meta);
+  Node* import_meta_const = jsgraph()->Constant(*import_meta, broker());
   ReplaceWithValue(node, import_meta_const);
   return Changed(import_meta_const);
 }
