@@ -16,34 +16,21 @@ class Isolate;
 // The incremental marking job uses platform tasks to perform incremental
 // marking steps. The job posts a foreground task that makes a small (~1ms)
 // step and posts another task until the marking is completed.
-class IncrementalMarkingJob {
+class IncrementalMarkingJob final {
  public:
-  enum class TaskType { kNormal, kDelayed };
+  explicit IncrementalMarkingJob(Heap* heap) V8_NOEXCEPT : heap_(heap) {}
 
-  IncrementalMarkingJob() V8_NOEXCEPT = default;
-
-  void Start(Heap* heap);
-
-  void ScheduleTask(Heap* heap, TaskType task_type = TaskType::kNormal);
+  void ScheduleTask();
+  double CurrentTimeToTask() const;
 
  private:
   class Task;
   static constexpr double kDelayInSeconds = 10.0 / 1000.0;
 
-  bool IsTaskPending(TaskType task_type) {
-    return task_type == TaskType::kNormal ? normal_task_pending_
-                                          : delayed_task_pending_;
-  }
-  void SetTaskPending(TaskType task_type, bool value) {
-    if (task_type == TaskType::kNormal) {
-      normal_task_pending_ = value;
-    } else {
-      delayed_task_pending_ = value;
-    }
-  }
-
-  bool normal_task_pending_ = false;
-  bool delayed_task_pending_ = false;
+  Heap* heap_;
+  base::Mutex mutex_;
+  double scheduled_time_ = 0.0;
+  bool is_task_pending_ = false;
 };
 }  // namespace internal
 }  // namespace v8
