@@ -5,10 +5,11 @@
 #ifndef V8_OBJECTS_FOREIGN_INL_H_
 #define V8_OBJECTS_FOREIGN_INL_H_
 
-#include "src/objects/foreign.h"
-
+#include "src/common/globals.h"
 #include "src/heap/heap-write-barrier-inl.h"
+#include "src/objects/foreign.h"
 #include "src/objects/objects-inl.h"
+#include "src/sandbox/external-pointer-inl.h"
 
 // Has to be the last include (doesn't have include guards):
 #include "src/objects/object-macros.h"
@@ -16,23 +17,12 @@
 namespace v8 {
 namespace internal {
 
-OBJECT_CONSTRUCTORS_IMPL(Foreign, HeapObject)
+#include "torque-generated/src/objects/foreign-tq-inl.inc"
 
-CAST_ACCESSOR(Foreign)
+TQ_OBJECT_CONSTRUCTORS_IMPL(Foreign)
 
-// static
-bool Foreign::IsNormalized(Object value) {
-  if (value == Smi::kZero) return true;
-  return Foreign::cast(value).foreign_address() != kNullAddress;
-}
-
-Address Foreign::foreign_address() {
-  return ReadField<Address>(kForeignAddressOffset);
-}
-
-void Foreign::set_foreign_address(Address value) {
-  WriteField<Address>(kForeignAddressOffset, value);
-}
+EXTERNAL_POINTER_ACCESSORS(Foreign, foreign_address, Address,
+                           kForeignAddressOffset, kForeignForeignAddressTag)
 
 }  // namespace internal
 }  // namespace v8
