@@ -15,6 +15,8 @@
 namespace v8 {
 namespace internal {
 
+#include "torque-generated/src/objects/literal-objects-tq-inl.inc"
+
 //
 // ObjectBoilerplateDescription
 //
@@ -27,24 +29,26 @@ SMI_ACCESSORS(ObjectBoilerplateDescription, flags,
               FixedArray::OffsetOfElementAt(kLiteralTypeOffset))
 
 Object ObjectBoilerplateDescription::name(int index) const {
-  Isolate* isolate = GetIsolateForPtrCompr(*this);
-  return name(isolate, index);
+  PtrComprCageBase cage_base = GetPtrComprCageBase(*this);
+  return name(cage_base, index);
 }
 
-Object ObjectBoilerplateDescription::name(Isolate* isolate, int index) const {
+Object ObjectBoilerplateDescription::name(PtrComprCageBase cage_base,
+                                          int index) const {
   // get() already checks for out of bounds access, but we do not want to allow
   // access to the last element, if it is the number of properties.
   DCHECK_NE(size(), index);
-  return get(isolate, 2 * index + kDescriptionStartIndex);
+  return get(cage_base, 2 * index + kDescriptionStartIndex);
 }
 
 Object ObjectBoilerplateDescription::value(int index) const {
-  Isolate* isolate = GetIsolateForPtrCompr(*this);
-  return value(isolate, index);
+  PtrComprCageBase cage_base = GetPtrComprCageBase(*this);
+  return value(cage_base, index);
 }
 
-Object ObjectBoilerplateDescription::value(Isolate* isolate, int index) const {
-  return get(isolate, 2 * index + 1 + kDescriptionStartIndex);
+Object ObjectBoilerplateDescription::value(PtrComprCageBase cage_base,
+                                           int index) const {
+  return get(cage_base, 2 * index + 1 + kDescriptionStartIndex);
 }
 
 void ObjectBoilerplateDescription::set_key_value(int index, Object key,
@@ -93,14 +97,8 @@ void ObjectBoilerplateDescription::set_backing_store_size(
 OBJECT_CONSTRUCTORS_IMPL(ClassBoilerplate, FixedArray)
 CAST_ACCESSOR(ClassBoilerplate)
 
-BIT_FIELD_ACCESSORS(ClassBoilerplate, flags, install_class_name_accessor,
-                    ClassBoilerplate::Flags::InstallClassNameAccessorBit)
-
-BIT_FIELD_ACCESSORS(ClassBoilerplate, flags, arguments_count,
-                    ClassBoilerplate::Flags::ArgumentsCountBits)
-
-SMI_ACCESSORS(ClassBoilerplate, flags,
-              FixedArray::OffsetOfElementAt(kFlagsIndex))
+SMI_ACCESSORS(ClassBoilerplate, arguments_count,
+              FixedArray::OffsetOfElementAt(kArgumentsCountIndex))
 
 ACCESSORS(ClassBoilerplate, static_properties_template, Object,
           FixedArray::OffsetOfElementAt(kClassPropertiesTemplateIndex))
@@ -126,8 +124,6 @@ ACCESSORS(ClassBoilerplate, instance_computed_properties, FixedArray,
 
 TQ_OBJECT_CONSTRUCTORS_IMPL(ArrayBoilerplateDescription)
 
-TQ_SMI_ACCESSORS(ArrayBoilerplateDescription, flags)
-
 ElementsKind ArrayBoilerplateDescription::elements_kind() const {
   return static_cast<ElementsKind>(flags());
 }
@@ -140,6 +136,12 @@ bool ArrayBoilerplateDescription::is_empty() const {
   return constant_elements().length() == 0;
 }
 
+//
+// RegExpBoilerplateDescription
+//
+
+TQ_OBJECT_CONSTRUCTORS_IMPL(RegExpBoilerplateDescription)
+
 }  // namespace internal
 }  // namespace v8
 
