@@ -2,6 +2,10 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#if !V8_ENABLE_WEBASSEMBLY
+#error This header should only be included if WebAssembly is enabled.
+#endif  // !V8_ENABLE_WEBASSEMBLY
+
 #ifndef V8_WASM_MODULE_COMPILER_H_
 #define V8_WASM_MODULE_COMPILER_H_
 
@@ -9,8 +13,14 @@
 #include <functional>
 #include <memory>
 
+#include "include/v8-metrics.h"
 #include "src/base/optional.h"
+#include "src/base/platform/elapsed-timer.h"
+#include "src/base/platform/mutex.h"
+#include "src/base/platform/time.h"
 #include "src/common/globals.h"
+#include "src/execution/isolate.h"
+#include "src/logging/counters.h"
 #include "src/tasks/cancelable-task.h"
 #include "src/wasm/compilation-environment.h"
 #include "src/wasm/wasm-features.h"
@@ -18,6 +28,12 @@
 #include "src/wasm/wasm-module.h"
 
 namespace v8 {
+
+namespace base {
+template <typename T>
+class Vector;
+}  // namespace base
+
 namespace internal {
 
 class JSArrayBuffer;
@@ -26,9 +42,6 @@ class Counters;
 class WasmModuleObject;
 class WasmInstanceObject;
 
-template <typename T>
-class Vector;
-
 namespace wasm {
 
 struct CompilationEnv;
@@ -36,47 +49,57 @@ class CompilationResultResolver;
 class ErrorThrower;
 class ModuleCompiler;
 class NativeModule;
+class ProfileInformation;
+class StreamingDecoder;
 class WasmCode;
 struct WasmModule;
 
+V8_EXPORT_PRIVATE
 std::shared_ptr<NativeModule> CompileToNativeModule(
-    Isolate* isolate, const WasmFeatures& enabled, ErrorThrower* thrower,
-    std::shared_ptr<const WasmModule> module, const ModuleWireBytes& wire_bytes,
-    Handle<FixedArray>* export_wrappers_out);
+    Isolate* isolate, WasmFeatures enabled_features, ErrorThrower* thrower,
+    std::shared_ptr<const WasmModule> module, ModuleWireBytes wire_bytes,
+    int compilation_id, v8::metrics::Recorder::ContextId context_id,
+    ProfileInformation* pgo_info);
 
 V8_EXPORT_PRIVATE
-void CompileJsToWasmWrappers(Isolate* isolate, const WasmModule* module,
-                             Handle<FixedArray>* export_wrappers_out);
+void CompileJsToWasmWrappers(Isolate* isolate, const WasmModule* module);
 
 // Compiles the wrapper for this (kind, sig) pair and sets the corresponding
 // cache entry. Assumes the key already exists in the cache but has not been
 // compiled yet.
 V8_EXPORT_PRIVATE
 WasmCode* CompileImportWrapper(
-    WasmEngine* wasm_engine, NativeModule* native_module, Counters* counters,
-    compiler::WasmImportCallKind kind, FunctionSig* sig,
-    WasmImportWrapperCache::ModificationScope* cache_scope);
-
-V8_EXPORT_PRIVATE Handle<Script> CreateWasmScript(
-    Isolate* isolate, const ModuleWireBytes& wire_bytes,
-    const std::string& source_map_url);
+    NativeModule* native_module, Counters* counters, ImportCallKind kind,
+    const FunctionSig* sig, uint32_t canonical_type_index, int expected_arity,
+    Suspend suspend, WasmImportWrapperCache::ModificationScope* cache_scope);
 
 // Triggered by the WasmCompileLazy builtin. The return value indicates whether
 // compilation was successful. Lazy compilation can fail only if validation is
 // also lazy.
-bool CompileLazy(Isolate*, NativeModule*, int func_index);
+bool CompileLazy(Isolate*, WasmInstanceObject, int func_index);
 
-int GetMaxBackgroundTasks();
+// Throws the compilation error after failed lazy compilation.
+void ThrowLazyCompilationError(Isolate* isolate,
+                               const NativeModule* native_module,
+                               int func_index);
 
-template <typename Key, typename Hash>
+// Trigger tier-up of a particular function to TurboFan. If tier-up was already
+// triggered, we instead increase the priority with exponential back-off.
+V8_EXPORT_PRIVATE void TriggerTierUp(WasmInstanceObject instance,
+                                     int func_index);
+// Synchronous version of the above.
+void TierUpNowForTesting(Isolate* isolate, WasmInstanceObject instance,
+                         int func_index);
+
+template <typename Key, typename KeyInfo, typename Hash>
 class WrapperQueue {
  public:
   // Removes an arbitrary key from the queue and returns it.
   // If the queue is empty, returns nullopt.
   // Thread-safe.
-  base::Optional<Key> pop() {
-    base::Optional<Key> key = base::nullopt;
-    base::LockGuard<base::Mutex> lock(&mutex_);
+  base::Optional<std::pair<Key, KeyInfo>> pop() {
+    base::Optional<std::pair<Key, KeyInfo>> key = base::nullopt;
+    base::MutexGuard lock(&mutex_);
     auto it = queue_.begin();
     if (it != queue_.end()) {
       key = *it;
@@ -88,11 +111,18 @@ class WrapperQueue {
   // Add the given key to the queue and returns true iff the insert was
   // successful.
   // Not thread-safe.
-  bool insert(const Key& key) { return queue_.insert(key).second; }
+  bool insert(const Key& key, KeyInfo key_info) {
+    return queue_.insert({key, key_info}).second;
+  }
+
+  size_t size() {
+    base::MutexGuard lock(&mutex_);
+    return queue_.size();
+  }
 
  private:
   base::Mutex mutex_;
-  std::unordered_set<Key, Hash> queue_;
+  std::unordered_map<Key, KeyInfo, Hash> queue_;
 };
 
 // Encapsulates all the state and steps of an asynchronous compilation.
@@ -104,10 +134,12 @@ class WrapperQueue {
 // TODO(wasm): factor out common parts of this with the synchronous pipeline.
 class AsyncCompileJob {
  public:
-  AsyncCompileJob(Isolate* isolate, const WasmFeatures& enabled_features,
-                  std::unique_ptr<byte[]> bytes_copy, size_t length,
-                  Handle<Context> context, const char* api_method_name,
-                  std::shared_ptr<CompilationResultResolver> resolver);
+  AsyncCompileJob(Isolate* isolate, WasmFeatures enabled_features,
+                  base::OwnedVector<const uint8_t> bytes,
+                  Handle<Context> context, Handle<Context> incumbent_context,
+                  const char* api_method_name,
+                  std::shared_ptr<CompilationResultResolver> resolver,
+                  int compilation_id);
   ~AsyncCompileJob();
 
   void Start();
@@ -119,7 +151,8 @@ class AsyncCompileJob {
 
   Isolate* isolate() const { return isolate_; }
 
-  Handle<Context> context() const { return native_context_; }
+  Handle<NativeContext> context() const { return native_context_; }
+  v8::metrics::Recorder::ContextId context_id() const { return context_id_; }
 
  private:
   class CompileTask;
@@ -127,33 +160,78 @@ class AsyncCompileJob {
   class CompilationStateCallback;
 
   // States of the AsyncCompileJob.
-  class DecodeModule;            // Step 1  (async)
-  class DecodeFail;              // Step 1b (sync)
-  class PrepareAndStartCompile;  // Step 2  (sync)
-  class CompileFailed;           // Step 3a (sync)
-  class CompileFinished;         // Step 3b (sync)
+  // Step 1 (async). Decodes the wasm module.
+  // --> Fail on decoding failure,
+  // --> PrepareAndStartCompile on success.
+  class DecodeModule;
+
+  // Step 2 (sync). Prepares runtime objects and starts background compilation.
+  // --> finish directly on native module cache hit,
+  // --> finish directly on validation error,
+  // --> trigger eager compilation, if any; FinishCompile is triggered when
+  // done.
+  class PrepareAndStartCompile;
+
+  // Step 3 (sync). Compilation finished. Finalize the module and resolve the
+  // promise.
+  class FinishCompilation;
+
+  // Step 4 (sync). Decoding, validation or compilation failed. Reject the
+  // promise.
+  class Fail;
 
   friend class AsyncStreamingProcessor;
 
+  enum FinishingComponent { kStreamingDecoder, kCompilation };
+
   // Decrements the number of outstanding finishers. The last caller of this
   // function should finish the asynchronous compilation, see the comment on
   // {outstanding_finishers_}.
-  V8_WARN_UNUSED_RESULT bool DecrementAndCheckFinisherCount() {
-    DCHECK_LT(0, outstanding_finishers_.load());
-    return outstanding_finishers_.fetch_sub(1) == 1;
+  V8_WARN_UNUSED_RESULT bool DecrementAndCheckFinisherCount(
+      FinishingComponent component) {
+    base::MutexGuard guard(&check_finisher_mutex_);
+    DCHECK_LT(0, outstanding_finishers_);
+    if (outstanding_finishers_-- == 2) {
+      // The first component finished, we just start a timer for a histogram.
+      streaming_until_finished_timer_.Start();
+      return false;
+    }
+    // The timer has only been started above in the case of streaming
+    // compilation.
+    if (streaming_until_finished_timer_.IsStarted()) {
+      // We measure the time delta from when the StreamingDecoder finishes until
+      // when module compilation finishes. Depending on whether streaming or
+      // compilation finishes first we add the delta to the according histogram.
+      int elapsed = static_cast<int>(
+          streaming_until_finished_timer_.Elapsed().InMilliseconds());
+      if (component == kStreamingDecoder) {
+        isolate_->counters()
+            ->wasm_compilation_until_streaming_finished()
+            ->AddSample(elapsed);
+      } else {
+        isolate_->counters()
+            ->wasm_streaming_until_compilation_finished()
+            ->AddSample(elapsed);
+      }
+    }
+    DCHECK_EQ(0, outstanding_finishers_);
+    return true;
   }
 
-  void CreateNativeModule(std::shared_ptr<const WasmModule> module);
+  void CreateNativeModule(std::shared_ptr<const WasmModule> module,
+                          size_t code_size_estimate);
+  // Return true for cache hit, false for cache miss.
+  bool GetOrCreateNativeModule(std::shared_ptr<const WasmModule> module,
+                               size_t code_size_estimate);
   void PrepareRuntimeObjects();
 
-  void FinishCompile();
+  void FinishCompile(bool is_after_cache_hit);
 
-  void DecodeFailed(const WasmError&);
-  void AsyncCompileFailed();
+  void Failed();
 
   void AsyncCompileSucceeded(Handle<WasmModuleObject> result);
 
-  void FinishModule();
+  void FinishSuccessfully();
 
   void StartForegroundTask();
   void ExecuteForegroundTaskImmediately();
@@ -190,14 +268,18 @@ class AsyncCompileJob {
   Isolate* const isolate_;
   const char* const api_method_name_;
   const WasmFeatures enabled_features_;
-  const bool wasm_lazy_compilation_;
+  const DynamicTiering dynamic_tiering_;
+  base::TimeTicks start_time_;
   // Copy of the module wire bytes, moved into the {native_module_} on its
   // creation.
-  std::unique_ptr<byte[]> bytes_copy_;
+  base::OwnedVector<const uint8_t> bytes_copy_;
   // Reference to the wire bytes (held in {bytes_copy_} or as part of
   // {native_module_}).
   ModuleWireBytes wire_bytes_;
-  Handle<Context> native_context_;
+  Handle<NativeContext> native_context_;
+  Handle<Context> incumbent_context_;
+  v8::metrics::Recorder::ContextId context_id_;
+  v8::metrics::WasmModuleDecoded metrics_event_;
   const std::shared_ptr<CompilationResultResolver> resolver_;
 
   Handle<WasmModuleObject> module_object_;
@@ -211,7 +293,9 @@ class AsyncCompileJob {
   // For async compilation the AsyncCompileJob is the only finisher. For
   // streaming compilation also the AsyncStreamingProcessor has to finish before
   // compilation can be finished.
-  std::atomic<int32_t> outstanding_finishers_{1};
+  int32_t outstanding_finishers_ = 1;
+  base::ElapsedTimer streaming_until_finished_timer_;
+  base::Mutex check_finisher_mutex_;
 
   // A reference to a pending foreground task, or {nullptr} if none is pending.
   CompileTask* pending_foreground_task_ = nullptr;
@@ -221,6 +305,9 @@ class AsyncCompileJob {
   // compilation. The AsyncCompileJob does not actively use the
   // StreamingDecoder.
   std::shared_ptr<StreamingDecoder> stream_;
+
+  // The compilation id to identify trace events linked to this compilation.
+  const int compilation_id_;
 };
 
 }  // namespace wasm
