@@ -4,13 +4,11 @@
 
 #include "src/wasm/wasm-result.h"
 
+#include "src/base/strings.h"
 #include "src/execution/isolate-inl.h"
 #include "src/heap/factory.h"
-#include "src/heap/heap.h"
 #include "src/objects/objects.h"
 
-#include "src/base/platform/platform.h"
-
 namespace v8 {
 namespace internal {
 namespace wasm {
@@ -28,9 +26,10 @@ void VPrintFToString(std::string* str, size_t str_offset, const char* format,
     str->resize(len);
     va_list args_copy;
     va_copy(args_copy, args);
-    int written = VSNPrintF(Vector<char>(&str->front() + str_offset,
-                                         static_cast<int>(len - str_offset)),
-                            format, args_copy);
+    int written =
+        base::VSNPrintF(base::Vector<char>(&str->front() + str_offset,
+                                           static_cast<int>(len - str_offset)),
+                        format, args_copy);
     va_end(args_copy);
     if (written < 0) continue;  // not enough space.
     str->resize(str_offset + written);
@@ -127,7 +126,7 @@ Handle<Object> ErrorThrower::Reify() {
       break;
   }
   Handle<String> message = isolate_->factory()
-                               ->NewStringFromUtf8(VectorOf(error_msg_))
+                               ->NewStringFromUtf8(base::VectorOf(error_msg_))
                                .ToHandleChecked();
   Reset();
   return isolate_->factory()->NewError(constructor, message);
@@ -155,6 +154,21 @@ ErrorThrower::~ErrorThrower() {
   }
 }
 
+ScheduledErrorThrower::~ScheduledErrorThrower() {
+  // There should never be both a pending and a scheduled exception.
+  DCHECK(!isolate()->has_scheduled_exception() ||
+         !isolate()->has_pending_exception());
+  // Don't throw another error if there is already a scheduled error.
+  if (isolate()->has_scheduled_exception()) {
+    Reset();
+  } else if (isolate()->has_pending_exception()) {
+    Reset();
+    isolate()->OptionalRescheduleException(false);
+  } else if (error()) {
+    isolate()->ScheduleThrow(*Reify());
+  }
+}
+
 }  // namespace wasm
 }  // namespace internal
 }  // namespace v8
