@@ -24,14 +24,11 @@
 const {
   ArrayIsArray,
   Error,
-  FunctionPrototypeCall,
   MathMin,
   ObjectKeys,
   ObjectSetPrototypeOf,
-  RegExpPrototypeExec,
   ReflectApply,
   Symbol,
-  SymbolAsyncDispose,
   SymbolFor,
 } = primordials;
 
@@ -84,6 +81,7 @@ const {
 const {
   kEmptyObject,
   promisify,
+  SymbolAsyncDispose,
 } = require('internal/util');
 const {
   validateInteger,
@@ -184,6 +182,8 @@ const kConnections = Symbol('http.server.connections');
 const kConnectionsCheckingInterval = Symbol('http.server.connectionsCheckingInterval');
 
 const HTTP_SERVER_TRACE_EVENT_NAME = 'http.server.request';
+// TODO(jazelly): make this configurable
+const HTTP_SERVER_KEEP_ALIVE_TIMEOUT_BUFFER = 1000;
 
 class HTTPServerAsyncResource {
   constructor(type, socket) {
@@ -203,8 +203,7 @@ function ServerResponse(req, options) {
   this._expect_continue = false;
 
   if (req.httpVersionMajor < 1 || req.httpVersionMinor < 1) {
-    this.useChunkedEncodingByDefault = RegExpPrototypeExec(chunkExpression,
-                                                           req.headers.te) !== null;
+    this.useChunkedEncodingByDefault = chunkExpression.test(req.headers.te);
     this.shouldKeepAlive = false;
   }
 
@@ -568,7 +567,7 @@ Server.prototype.close = function() {
 };
 
 Server.prototype[SymbolAsyncDispose] = async function() {
-  return FunctionPrototypeCall(promisify(this.close), this);
+  return promisify(this.close).call(this);
 };
 
 Server.prototype.closeAllConnections = function() {
@@ -1001,7 +1000,9 @@ function resOnFinish(req, res, socket, state, server) {
     }
   } else if (state.outgoing.length === 0) {
     if (server.keepAliveTimeout && typeof socket.setTimeout === 'function') {
-      socket.setTimeout(server.keepAliveTimeout);
+      // Increase the internal timeout wrt the advertised value to reduce
+      // the likelihood of ECONNRESET errors.
+      socket.setTimeout(server.keepAliveTimeout + HTTP_SERVER_KEEP_ALIVE_TIMEOUT_BUFFER);
       state.keepAliveTimeoutSet = true;
     }
   } else {
@@ -1119,7 +1120,7 @@ function parserOnIncoming(server, socket, state, req, keepAlive) {
     } else if (req.headers.expect !== undefined) {
       handled = true;
 
-      if (RegExpPrototypeExec(continueExpression, req.headers.expect) !== null) {
+      if (continueExpression.test(req.headers.expect)) {
         res._expect_continue = true;
         if (server.listenerCount('checkContinue') > 0) {
           server.emit('checkContinue', req, res);
