@@ -23,17 +23,15 @@
 
 const {
   Array,
-  ArrayFrom,
   ArrayIsArray,
   ArrayPrototypeForEach,
-  ArrayPrototypeIndexOf,
   MathFloor,
   MathMin,
   MathTrunc,
+  NumberIsInteger,
   NumberIsNaN,
   NumberMAX_SAFE_INTEGER,
   NumberMIN_SAFE_INTEGER,
-  ObjectCreate,
   ObjectDefineProperties,
   ObjectDefineProperty,
   ObjectSetPrototypeOf,
@@ -59,7 +57,6 @@ const {
   byteLengthUtf8,
   compare: _compare,
   compareOffset,
-  createFromString,
   fill: bindingFill,
   isAscii: bindingIsAscii,
   isUtf8: bindingIsUtf8,
@@ -71,6 +68,8 @@ const {
   swap64: _swap64,
   kMaxLength,
   kStringMaxLength,
+  atob: _atob,
+  btoa: _btoa,
 } = internalBinding('buffer');
 const {
   constants: {
@@ -78,14 +77,15 @@ const {
     ONLY_ENUMERABLE,
   },
   getOwnNonIndexProperties,
+  isInsideNodeModules,
 } = internalBinding('util');
 const {
   customInspectSymbol,
-  isInsideNodeModules,
   lazyDOMException,
   normalizeEncoding,
   kIsEncodingSymbol,
   defineLazyProperties,
+  encodingsMap,
 } = require('internal/util');
 const {
   isAnyArrayBuffer,
@@ -96,7 +96,6 @@ const {
 const {
   inspect: utilInspect,
 } = require('internal/util/inspect');
-const { encodings } = internalBinding('string_decoder');
 
 const {
   codes: {
@@ -109,13 +108,12 @@ const {
     ERR_UNKNOWN_ENCODING,
   },
   genericNodeError,
-  hideStackFrames,
 } = require('internal/errors');
 const {
   validateArray,
   validateBuffer,
-  validateNumber,
   validateInteger,
+  validateNumber,
   validateString,
 } = require('internal/validators');
 // Provide validateInteger() but with kMaxLength as the default maximum value.
@@ -149,15 +147,12 @@ const constants = ObjectDefineProperties({}, {
 });
 
 Buffer.poolSize = 8 * 1024;
-let poolSize, poolOffset, allocPool;
-
-const encodingsMap = ObjectCreate(null);
-for (let i = 0; i < encodings.length; ++i)
-  encodingsMap[encodings[i]] = i;
+let poolSize, poolOffset, allocPool, allocBuffer;
 
 function createPool() {
   poolSize = Buffer.poolSize;
-  allocPool = createUnsafeBuffer(poolSize).buffer;
+  allocBuffer = createUnsafeBuffer(poolSize);
+  allocPool = allocBuffer.buffer;
   markAsUntransferable(allocPool);
   poolOffset = 0;
 }
@@ -181,13 +176,15 @@ function showFlaggedDeprecation() {
   if (bufferWarningAlreadyEmitted ||
       ++nodeModulesCheckCounter > 10000 ||
       (!require('internal/options').getOptionValue('--pending-deprecation') &&
-       isInsideNodeModules())) {
+       isInsideNodeModules(100, true))) {
     // We don't emit a warning, because we either:
     // - Already did so, or
     // - Already checked too many times whether a call is coming
     //   from node_modules and want to stop slowing down things, or
     // - We aren't running with `--pending-deprecation` enabled,
     //   and the code is inside `node_modules`.
+    // - We found node_modules in up to the topmost 100 frames, or
+    //   there are more than 100 frames and we don't want to search anymore.
     return;
   }
 
@@ -214,7 +211,7 @@ function _copy(source, target, targetStart, sourceStart, sourceEnd) {
   if (targetStart === undefined) {
     targetStart = 0;
   } else {
-    targetStart = toInteger(targetStart, 0);
+    targetStart = NumberIsInteger(targetStart) ? targetStart : toInteger(targetStart, 0);
     if (targetStart < 0)
       throw new ERR_OUT_OF_RANGE('targetStart', '>= 0', targetStart);
   }
@@ -222,7 +219,7 @@ function _copy(source, target, targetStart, sourceStart, sourceEnd) {
   if (sourceStart === undefined) {
     sourceStart = 0;
   } else {
-    sourceStart = toInteger(sourceStart, 0);
+    sourceStart = NumberIsInteger(sourceStart) ? sourceStart : toInteger(sourceStart, 0);
     if (sourceStart < 0 || sourceStart > source.length)
       throw new ERR_OUT_OF_RANGE('sourceStart', `>= 0 && <= ${source.length}`, sourceStart);
   }
@@ -230,7 +227,7 @@ function _copy(source, target, targetStart, sourceStart, sourceEnd) {
   if (sourceEnd === undefined) {
     sourceEnd = source.length;
   } else {
-    sourceEnd = toInteger(sourceEnd, 0);
+    sourceEnd = NumberIsInteger(sourceEnd) ? sourceEnd : toInteger(sourceEnd, 0);
     if (sourceEnd < 0)
       throw new ERR_OUT_OF_RANGE('sourceEnd', '>= 0', sourceEnd);
   }
@@ -387,22 +384,12 @@ Buffer.of = of;
 
 ObjectSetPrototypeOf(Buffer, Uint8Array);
 
-// The 'assertSize' method will remove itself from the callstack when an error
-// occurs. This is done simply to keep the internal details of the
-// implementation from bleeding out to users.
-const assertSize = hideStackFrames((size) => {
-  validateNumber(size, 'size');
-  if (!(size >= 0 && size <= kMaxLength)) {
-    throw new ERR_INVALID_ARG_VALUE.RangeError('size', size);
-  }
-});
-
 /**
  * Creates a new filled Buffer instance.
  * alloc(size[, fill[, encoding]])
  */
 Buffer.alloc = function alloc(size, fill, encoding) {
-  assertSize(size);
+  validateNumber(size, 'size', 0, kMaxLength);
   if (fill !== undefined && fill !== 0 && size > 0) {
     const buf = createUnsafeBuffer(size);
     return _fill(buf, fill, 0, buf.length, encoding);
@@ -415,7 +402,7 @@ Buffer.alloc = function alloc(size, fill, encoding) {
  * instance. If `--zero-fill-buffers` is set, will zero-fill the buffer.
  */
 Buffer.allocUnsafe = function allocUnsafe(size) {
-  assertSize(size);
+  validateNumber(size, 'size', 0, kMaxLength);
   return allocate(size);
 };
 
@@ -425,15 +412,15 @@ Buffer.allocUnsafe = function allocUnsafe(size) {
  * If `--zero-fill-buffers` is set, will zero-fill the buffer.
  */
 Buffer.allocUnsafeSlow = function allocUnsafeSlow(size) {
-  assertSize(size);
+  validateNumber(size, 'size', 0, kMaxLength);
   return createUnsafeBuffer(size);
 };
 
 // If --zero-fill-buffers command line argument is set, a zero-filled
 // buffer is returned.
-function SlowBuffer(length) {
-  assertSize(length);
-  return createUnsafeBuffer(length);
+function SlowBuffer(size) {
+  validateNumber(size, 'size', 0, kMaxLength);
+  return createUnsafeBuffer(size);
 }
 
 ObjectSetPrototypeOf(SlowBuffer.prototype, Uint8ArrayPrototype);
@@ -455,39 +442,49 @@ function allocate(size) {
 }
 
 function fromStringFast(string, ops) {
-  const length = ops.byteLength(string);
+  const maxLength = Buffer.poolSize >>> 1;
 
-  if (length >= (Buffer.poolSize >>> 1))
-    return createFromString(string, ops.encodingVal);
+  let length = string.length; // Min length
+
+  if (length >= maxLength)
+    return createFromString(string, ops);
+
+  length *= 4; // Max length (4 bytes per character)
+
+  if (length >= maxLength)
+    length = ops.byteLength(string); // Actual length
+
+  if (length >= maxLength)
+    return createFromString(string, ops, length);
 
   if (length > (poolSize - poolOffset))
     createPool();
-  let b = new FastBuffer(allocPool, poolOffset, length);
-  const actual = ops.write(b, string, 0, length);
-  if (actual !== length) {
-    // byteLength() may overestimate. That's a rare case, though.
-    b = new FastBuffer(allocPool, poolOffset, actual);
-  }
+
+  const actual = ops.write(allocBuffer, string, poolOffset, length);
+  const b = new FastBuffer(allocPool, poolOffset, actual);
+
   poolOffset += actual;
   alignPool();
   return b;
 }
 
+function createFromString(string, ops, length = ops.byteLength(string)) {
+  const buf = Buffer.allocUnsafeSlow(length);
+  const actual = ops.write(buf, string, 0, length);
+  return actual < length ? new FastBuffer(buf.buffer, 0, actual) : buf;
+}
+
 function fromString(string, encoding) {
   let ops;
-  if (typeof encoding !== 'string' || encoding.length === 0) {
-    if (string.length === 0)
-      return new FastBuffer();
+  if (!encoding || encoding === 'utf8' || typeof encoding !== 'string') {
     ops = encodingOps.utf8;
-    encoding = undefined;
   } else {
     ops = getEncodingOps(encoding);
     if (ops === undefined)
       throw new ERR_UNKNOWN_ENCODING(encoding);
-    if (string.length === 0)
-      return new FastBuffer();
   }
-  return fromStringFast(string, ops);
+
+  return string.length === 0 ? new FastBuffer() : fromStringFast(string, ops);
 }
 
 function fromArrayBuffer(obj, byteOffset, length) {
@@ -785,13 +782,22 @@ function byteLength(string, encoding) {
   if (len === 0)
     return 0;
 
-  if (encoding) {
-    const ops = getEncodingOps(encoding);
-    if (ops) {
-      return ops.byteLength(string);
-    }
+  if (!encoding || encoding === 'utf8') {
+    return byteLengthUtf8(string);
   }
-  return byteLengthUtf8(string);
+
+  if (encoding === 'ascii') {
+    return len;
+  }
+
+  const ops = getEncodingOps(encoding);
+  if (ops === undefined) {
+    // TODO (ronag): Makes more sense to throw here.
+    // throw new ERR_UNKNOWN_ENCODING(encoding);
+    return byteLengthUtf8(string);
+  }
+
+  return ops.byteLength(string);
 }
 
 Buffer.byteLength = byteLength;
@@ -837,12 +843,12 @@ Buffer.prototype.toString = function toString(encoding, start, end) {
   else if (start >= len)
     return '';
   else
-    start |= 0;
+    start = MathTrunc(start) || 0;
 
   if (end === undefined || end > len)
     end = len;
   else
-    end |= 0;
+    end = MathTrunc(end) || 0;
 
   if (end <= start)
     return '';
@@ -865,11 +871,11 @@ Buffer.prototype.equals = function equals(otherBuffer) {
 
   if (this === otherBuffer)
     return true;
-
-  if (this.byteLength !== otherBuffer.byteLength)
+  const len = TypedArrayPrototypeGetByteLength(this);
+  if (len !== TypedArrayPrototypeGetByteLength(otherBuffer))
     return false;
 
-  return this.byteLength === 0 || _compare(this, otherBuffer) === 0;
+  return len === 0 || _compare(this, otherBuffer) === 0;
 };
 
 let INSPECT_MAX_BYTES = 50;
@@ -886,7 +892,7 @@ Buffer.prototype[customInspectSymbol] = function inspect(recurseTimes, ctx) {
   if (ctx) {
     let extras = false;
     const filter = ctx.showHidden ? ALL_PROPERTIES : ONLY_ENUMERABLE;
-    const obj = ObjectCreate(null);
+    const obj = { __proto__: null };
     ArrayPrototypeForEach(getOwnNonIndexProperties(this, filter),
                           (key) => {
                             extras = true;
@@ -1122,8 +1128,10 @@ Buffer.prototype.write = function write(string, offset, length, encoding) {
     }
   }
 
-  if (!encoding)
+  if (!encoding || encoding === 'utf8')
     return this.utf8Write(string, offset, length);
+  if (encoding === 'ascii')
+    return this.asciiWrite(string, offset, length);
 
   const ops = getEncodingOps(encoding);
   if (ops === undefined)
@@ -1267,94 +1275,33 @@ function btoa(input) {
   if (arguments.length === 0) {
     throw new ERR_MISSING_ARGS('input');
   }
-  input = `${input}`;
-  for (let n = 0; n < input.length; n++) {
-    if (input[n].charCodeAt(0) > 0xff)
-      throw lazyDOMException('Invalid character', 'InvalidCharacterError');
+  const result = _btoa(`${input}`);
+  if (result === -1) {
+    throw lazyDOMException('Invalid character', 'InvalidCharacterError');
   }
-  const buf = Buffer.from(input, 'latin1');
-  return buf.toString('base64');
+  return result;
 }
 
-// Refs: https://infra.spec.whatwg.org/#forgiving-base64-decode
-const kForgivingBase64AllowedChars = [
-  // ASCII whitespace
-  // Refs: https://infra.spec.whatwg.org/#ascii-whitespace
-  0x09, 0x0A, 0x0C, 0x0D, 0x20,
-
-  // Uppercase letters
-  ...ArrayFrom({ length: 26 }, (_, i) => StringPrototypeCharCodeAt('A') + i),
-
-  // Lowercase letters
-  ...ArrayFrom({ length: 26 }, (_, i) => StringPrototypeCharCodeAt('a') + i),
-
-  // Decimal digits
-  ...ArrayFrom({ length: 10 }, (_, i) => StringPrototypeCharCodeAt('0') + i),
-
-  0x2B, // +
-  0x2F, // /
-  0x3D, // =
-];
-const kEqualSignIndex = ArrayPrototypeIndexOf(kForgivingBase64AllowedChars,
-                                              0x3D);
-
 function atob(input) {
-  // The implementation here has not been performance optimized in any way and
-  // should not be.
-  // Refs: https://github.com/nodejs/node/pull/38433#issuecomment-828426932
   if (arguments.length === 0) {
     throw new ERR_MISSING_ARGS('input');
   }
 
-  input = `${input}`;
-  let nonAsciiWhitespaceCharCount = 0;
-  let equalCharCount = 0;
+  const result = _atob(`${input}`);
 
-  for (let n = 0; n < input.length; n++) {
-    const index = ArrayPrototypeIndexOf(
-      kForgivingBase64AllowedChars,
-      StringPrototypeCharCodeAt(input, n));
-
-    if (index > 4) {
-      // The first 5 elements of `kForgivingBase64AllowedChars` are
-      // ASCII whitespace char codes.
-      nonAsciiWhitespaceCharCount++;
-
-      if (index === kEqualSignIndex) {
-        equalCharCount++;
-      } else if (equalCharCount) {
-        // The `=` char is only allowed at the end.
-        throw lazyDOMException('Invalid character', 'InvalidCharacterError');
-      }
-
-      if (equalCharCount > 2) {
-        // Only one more `=` is permitted after the first equal sign.
-        throw lazyDOMException('Invalid character', 'InvalidCharacterError');
-      }
-    } else if (index === -1) {
+  switch (result) {
+    case -2: // Invalid character
       throw lazyDOMException('Invalid character', 'InvalidCharacterError');
-    }
+    case -1: // Single character remained
+      throw lazyDOMException(
+        'The string to be decoded is not correctly encoded.',
+        'InvalidCharacterError');
+    case -3: // Possible overflow
+      // TODO(@anonrig): Throw correct error in here.
+      throw lazyDOMException('The input causes overflow.', 'InvalidCharacterError');
+    default:
+      return result;
   }
-
-  let reminder = nonAsciiWhitespaceCharCount % 4;
-
-  // See #2, #3, #4 - https://infra.spec.whatwg.org/#forgiving-base64
-  if (!reminder) {
-    // Remove all trailing `=` characters and get the new reminder.
-    reminder = (nonAsciiWhitespaceCharCount - equalCharCount) % 4;
-  } else if (equalCharCount) {
-    // `=` should not in the input if there's a reminder.
-    throw lazyDOMException('Invalid character', 'InvalidCharacterError');
-  }
-
-  // See #3 - https://infra.spec.whatwg.org/#forgiving-base64
-  if (reminder === 1) {
-    throw lazyDOMException(
-      'The string to be decoded is not correctly encoded.',
-      'InvalidCharacterError');
-  }
-
-  return Buffer.from(input, 'base64').toString('latin1');
 }
 
 function isUtf8(input) {
@@ -1362,7 +1309,7 @@ function isUtf8(input) {
     return bindingIsUtf8(input);
   }
 
-  throw new ERR_INVALID_ARG_TYPE('input', ['TypedArray', 'Buffer'], input);
+  throw new ERR_INVALID_ARG_TYPE('input', ['ArrayBuffer', 'Buffer', 'TypedArray'], input);
 }
 
 function isAscii(input) {
@@ -1399,7 +1346,10 @@ ObjectDefineProperties(module.exports, {
     configurable: true,
     enumerable: true,
     get() { return INSPECT_MAX_BYTES; },
-    set(val) { INSPECT_MAX_BYTES = val; },
+    set(val) {
+      validateNumber(val, 'INSPECT_MAX_BYTES', 0);
+      INSPECT_MAX_BYTES = val;
+    },
   },
 });
 
