@@ -1,7 +1,6 @@
 'use strict';
 
 const {
-  ArrayFrom,
   ArrayPrototypeSlice,
   ObjectDefineProperty,
   ObjectDefineProperties,
@@ -687,7 +686,7 @@ class CryptoKey {
   get usages() {
     if (!(this instanceof CryptoKey))
       throw new ERR_INVALID_THIS('CryptoKey');
-    return ArrayFrom(this[kKeyUsages]);
+    return this[kKeyUsages];
   }
 }
 
@@ -703,24 +702,73 @@ ObjectDefineProperties(CryptoKey.prototype, {
   },
 });
 
+/**
+ * @param {InternalCryptoKey} key
+ * @param {KeyObject} keyObject
+ * @param {object} algorithm
+ * @param {boolean} extractable
+ * @param {Set<string>} keyUsages
+ */
+function defineCryptoKeyProperties(
+  key,
+  keyObject,
+  algorithm,
+  extractable,
+  keyUsages,
+) {
+  // Using symbol properties here currently instead of private
+  // properties because (for now) the performance penalty of
+  // private fields is still too high.
+  ObjectDefineProperties(key, {
+    [kKeyObject]: {
+      __proto__: null,
+      value: keyObject,
+      enumerable: false,
+      configurable: false,
+      writable: false,
+    },
+    [kAlgorithm]: {
+      __proto__: null,
+      value: algorithm,
+      enumerable: false,
+      configurable: false,
+      writable: false,
+    },
+    [kExtractable]: {
+      __proto__: null,
+      value: extractable,
+      enumerable: false,
+      configurable: false,
+      writable: false,
+    },
+    [kKeyUsages]: {
+      __proto__: null,
+      value: keyUsages,
+      enumerable: false,
+      configurable: false,
+      writable: false,
+    },
+  });
+}
+
 // All internal code must use new InternalCryptoKey to create
 // CryptoKey instances. The CryptoKey class is exposed to end
 // user code but is not permitted to be constructed directly.
 // Using makeTransferable also allows the CryptoKey to be
 // cloned to Workers.
 class InternalCryptoKey {
-  constructor(
-    keyObject,
-    algorithm,
-    keyUsages,
-    extractable) {
-    // Using symbol properties here currently instead of private
-    // properties because (for now) the performance penalty of
-    // private fields is still too high.
-    this[kKeyObject] = keyObject;
-    this[kAlgorithm] = algorithm;
-    this[kExtractable] = extractable;
-    this[kKeyUsages] = keyUsages;
+  constructor(keyObject, algorithm, keyUsages, extractable) {
+    // When constructed during transfer the properties get assigned
+    // in the kDeserialize call.
+    if (keyObject) {
+      defineCryptoKeyProperties(
+        this,
+        keyObject,
+        algorithm,
+        extractable,
+        keyUsages,
+      );
+    }
 
     // eslint-disable-next-line no-constructor-return
     return makeTransferable(this);
@@ -728,9 +776,9 @@ class InternalCryptoKey {
 
   [kClone]() {
     const keyObject = this[kKeyObject];
-    const algorithm = this.algorithm;
-    const extractable = this.extractable;
-    const usages = this.usages;
+    const algorithm = this[kAlgorithm];
+    const extractable = this[kExtractable];
+    const usages = this[kKeyUsages];
 
     return {
       data: {
@@ -744,10 +792,7 @@ class InternalCryptoKey {
   }
 
   [kDeserialize]({ keyObject, algorithm, usages, extractable }) {
-    this[kKeyObject] = keyObject;
-    this[kAlgorithm] = algorithm;
-    this[kKeyUsages] = usages;
-    this[kExtractable] = extractable;
+    defineCryptoKeyProperties(this, keyObject, algorithm, extractable, usages);
   }
 }
 InternalCryptoKey.prototype.constructor = CryptoKey;
