@@ -12,7 +12,6 @@ const {
   createResolverClass,
   validateHints,
   emitInvalidHostnameWarning,
-  getDefaultVerbatim,
   errorCodes: dnsErrorCodes,
   getDefaultResultOrder,
   setDefaultResultOrder,
@@ -45,7 +44,7 @@ const {
   ADDRGETNETWORKPARAMS,
   CANCELLED,
 } = dnsErrorCodes;
-const { codes, dnsException } = require('internal/errors');
+const { codes, DNSException } = require('internal/errors');
 const { isIP } = require('internal/net');
 const {
   getaddrinfo,
@@ -53,6 +52,9 @@ const {
   GetAddrInfoReqWrap,
   GetNameInfoReqWrap,
   QueryReqWrap,
+  DNS_ORDER_VERBATIM,
+  DNS_ORDER_IPV4_FIRST,
+  DNS_ORDER_IPV6_FIRST,
 } = internalBinding('cares_wrap');
 const {
   ERR_INVALID_ARG_TYPE,
@@ -79,7 +81,7 @@ const {
 
 function onlookup(err, addresses) {
   if (err) {
-    this.reject(dnsException(err, 'getaddrinfo', this.hostname));
+    this.reject(new DNSException(err, 'getaddrinfo', this.hostname));
     return;
   }
 
@@ -92,7 +94,7 @@ function onlookup(err, addresses) {
 
 function onlookupall(err, addresses) {
   if (err) {
-    this.reject(dnsException(err, 'getaddrinfo', this.hostname));
+    this.reject(new DNSException(err, 'getaddrinfo', this.hostname));
     return;
   }
 
@@ -120,13 +122,13 @@ function onlookupall(err, addresses) {
  * @param {boolean} all - Whether to resolve with all IP addresses for the hostname.
  * @param {number} hints - One or more supported getaddrinfo flags (supply multiple via
  * bitwise OR).
- * @param {boolean} verbatim - Whether to use the hostname verbatim.
+ * @param {number} dnsOrder - How to sort results. Must be `ipv4first`, `ipv6first` or `verbatim`.
  * @returns {Promise<DNSLookupResult | DNSLookupResult[]>} The IP address(es) of the hostname.
  * @typedef {object} DNSLookupResult
  * @property {string} address - The IP address.
  * @property {0 | 4 | 6} family - The IP address type. 4 for IPv4 or 6 for IPv6, or 0 (for both).
  */
-function createLookupPromise(family, hostname, all, hints, verbatim) {
+function createLookupPromise(family, hostname, all, hints, dnsOrder) {
   return new Promise((resolve, reject) => {
     if (!hostname) {
       emitInvalidHostnameWarning(hostname);
@@ -150,16 +152,25 @@ function createLookupPromise(family, hostname, all, hints, verbatim) {
     req.resolve = resolve;
     req.reject = reject;
 
-    const err = getaddrinfo(req, hostname, family, hints, verbatim);
+    let order = DNS_ORDER_VERBATIM;
+
+    if (dnsOrder === 'ipv4first') {
+      order = DNS_ORDER_IPV4_FIRST;
+    } else if (dnsOrder === 'ipv6first') {
+      order = DNS_ORDER_IPV6_FIRST;
+    }
+
+    const err = getaddrinfo(req, hostname, family, hints, order);
 
     if (err) {
-      reject(dnsException(err, 'getaddrinfo', hostname));
+      reject(new DNSException(err, 'getaddrinfo', hostname));
     } else if (hasObserver('dns')) {
       const detail = {
         hostname,
         family,
         hints,
-        verbatim,
+        verbatim: order === DNS_ORDER_VERBATIM,
+        order: dnsOrder,
       };
       startPerf(req, kPerfHooksDnsLookupContext, { type: 'dns', name: 'lookup', detail });
     }
@@ -175,14 +186,15 @@ const validFamilies = [0, 4, 6];
  * @param {0 | 4 | 6} [options.family=0] - The record family. Must be 4, 6, or 0 (for both).
  * @param {number} [options.hints] - One or more supported getaddrinfo flags (supply multiple via
  * bitwise OR).
- * @param {boolean} [options.verbatim=false] - Return results in same order DNS resolved them;
- * otherwise IPv4 then IPv6. New code should supply `true`.
+ * @param {string} [options.order='verbatim'] - Return results in same order DNS resolved them;
+ * Must be `ipv4first`, `ipv6first` or `verbatim`.
+ * New code should supply `verbatim`.
  */
 function lookup(hostname, options) {
   let hints = 0;
   let family = 0;
   let all = false;
-  let verbatim = getDefaultVerbatim();
+  let dnsOrder = getDefaultResultOrder();
 
   // Parse arguments
   if (hostname) {
@@ -210,17 +222,21 @@ function lookup(hostname, options) {
     }
     if (options?.verbatim != null) {
       validateBoolean(options.verbatim, 'options.verbatim');
-      verbatim = options.verbatim;
+      dnsOrder = options.verbatim ? 'verbatim' : 'ipv4first';
+    }
+    if (options?.order != null) {
+      validateOneOf(options.order, 'options.order', ['ipv4first', 'ipv6first', 'verbatim']);
+      dnsOrder = options.order;
     }
   }
 
-  return createLookupPromise(family, hostname, all, hints, verbatim);
+  return createLookupPromise(family, hostname, all, hints, dnsOrder);
 }
 
 
 function onlookupservice(err, hostname, service) {
   if (err) {
-    this.reject(dnsException(err, 'getnameinfo', this.host));
+    this.reject(new DNSException(err, 'getnameinfo', this.host));
     return;
   }
 
@@ -243,7 +259,7 @@ function createLookupServicePromise(hostname, port) {
     const err = getnameinfo(req, hostname, port);
 
     if (err)
-      reject(dnsException(err, 'getnameinfo', hostname));
+      reject(new DNSException(err, 'getnameinfo', hostname));
     else if (hasObserver('dns')) {
       startPerf(req, kPerfHooksDnsLookupServiceContext, {
         type: 'dns',
@@ -272,7 +288,7 @@ function lookupService(address, port) {
 
 function onresolve(err, result, ttls) {
   if (err) {
-    this.reject(dnsException(err, this.bindingName, this.hostname));
+    this.reject(new DNSException(err, this.bindingName, this.hostname));
     return;
   }
 
@@ -300,7 +316,7 @@ function createResolverPromise(resolver, bindingName, hostname, ttl) {
     const err = resolver._handle[bindingName](req, hostname);
 
     if (err)
-      reject(dnsException(err, bindingName, hostname));
+      reject(new DNSException(err, bindingName, hostname));
     else if (hasObserver('dns')) {
       startPerf(req, kPerfHooksDnsLookupResolveContext, {
         type: 'dns',
