@@ -5,6 +5,7 @@ const {
   ObjectPrototypeHasOwnProperty,
   PromisePrototypeThen,
   PromiseResolve,
+  SafeSet,
   StringPrototypeIncludes,
   StringPrototypeCharCodeAt,
   StringPrototypeSlice,
@@ -13,18 +14,14 @@ const { getOptionValue } = require('internal/options');
 const {
   extensionFormatMap,
   getFormatOfExtensionlessFile,
-  getLegacyExtensionFormat,
   mimeToFormat,
 } = require('internal/modules/esm/formats');
 
+const detectModule = getOptionValue('--experimental-detect-module');
 const experimentalNetworkImports =
   getOptionValue('--experimental-network-imports');
-const experimentalSpecifierResolution =
-  getOptionValue('--experimental-specifier-resolution');
-const defaultTypeFlag = getOptionValue('--experimental-default-type');
-// The next line is where we flip the default to ES modules someday.
-const defaultType = defaultTypeFlag === 'module' ? 'module' : 'commonjs';
-const { getPackageType } = require('internal/modules/esm/resolve');
+const { containsModuleSyntax } = internalBinding('contextify');
+const { getPackageScopeConfig, getPackageType } = require('internal/modules/esm/resolve');
 const { fileURLToPath } = require('internal/url');
 const { ERR_UNKNOWN_FILE_EXTENSION } = require('internal/errors').codes;
 
@@ -37,6 +34,17 @@ const protocolHandlers = {
   'node:'() { return 'builtin'; },
 };
 
+/**
+ * Determine whether the given ambiguous source contains CommonJS or ES module syntax.
+ * @param {string | Buffer | undefined} source
+ * @param {URL} url
+ */
+function detectModuleFormat(source, url) {
+  if (!source) { return detectModule ? null : 'commonjs'; }
+  if (!detectModule) { return 'commonjs'; }
+  return containsModuleSyntax(`${source}`, fileURLToPath(url), url) ? 'module' : 'commonjs';
+}
+
 /**
  * @param {URL} parsed
  * @returns {string | null}
@@ -86,57 +94,99 @@ function underNodeModules(url) {
   return StringPrototypeIncludes(url.pathname, '/node_modules/');
 }
 
+let typelessPackageJsonFilesWarnedAbout;
+function warnTypelessPackageJsonFile(pjsonPath, url) {
+  typelessPackageJsonFilesWarnedAbout ??= new SafeSet();
+  if (!underNodeModules(url) && !typelessPackageJsonFilesWarnedAbout.has(pjsonPath)) {
+    const warning = `Module type of ${url} is not specified and it doesn't parse as CommonJS.\n` +
+      'Reparsing as ES module because module syntax was detected. This incurs a performance overhead.\n' +
+      `To eliminate this warning, add "type": "module" to ${pjsonPath}.`;
+    process.emitWarning(warning, {
+      code: 'MODULE_TYPELESS_PACKAGE_JSON',
+    });
+    typelessPackageJsonFilesWarnedAbout.add(pjsonPath);
+  }
+}
+
 /**
  * @param {URL} url
- * @param {{parentURL: string}} context
+ * @param {{parentURL: string; source?: Buffer}} context
  * @param {boolean} ignoreErrors
  * @returns {string}
  */
-function getFileProtocolModuleFormat(url, context, ignoreErrors) {
+function getFileProtocolModuleFormat(url, context = { __proto__: null }, ignoreErrors) {
+  const { source } = context;
   const ext = extname(url);
 
   if (ext === '.js') {
-    const packageType = getPackageType(url);
+    const { type: packageType, pjsonPath, exists: foundPackageJson } = getPackageScopeConfig(url);
     if (packageType !== 'none') {
       return packageType;
     }
+
     // The controlling `package.json` file has no `type` field.
-    if (defaultType === 'module') {
-      // An exception to the type flag making ESM the default everywhere is that package scopes under `node_modules`
-      // should retain the assumption that a lack of a `type` field means CommonJS.
-      return underNodeModules(url) ? 'commonjs' : 'module';
+    switch (getOptionValue('--experimental-default-type')) {
+      case 'module': { // The user explicitly passed `--experimental-default-type=module`.
+        // An exception to the type flag making ESM the default everywhere is that package scopes under `node_modules`
+        // should retain the assumption that a lack of a `type` field means CommonJS.
+        return underNodeModules(url) ? 'commonjs' : 'module';
+      }
+      case 'commonjs': { // The user explicitly passed `--experimental-default-type=commonjs`.
+        return 'commonjs';
+      }
+      default: { // The user did not pass `--experimental-default-type`.
+        // `source` is undefined when this is called from `defaultResolve`;
+        // but this gets called again from `defaultLoad`/`defaultLoadSync`.
+        // For ambiguous files (no type field, .js extension) we return
+        // undefined from `resolve` and re-run the check in `load`.
+        const format = detectModuleFormat(source, url);
+        if (format === 'module' && foundPackageJson) {
+          // This module has a .js extension, a package.json with no `type` field, and ESM syntax.
+          // Warn about the missing `type` field so that the user can avoid the performance penalty of detection.
+          warnTypelessPackageJsonFile(pjsonPath, url);
+        }
+        return format;
+      }
     }
-    return 'commonjs';
   }
 
   if (ext === '') {
     const packageType = getPackageType(url);
-    if (defaultType === 'commonjs') { // Legacy behavior
-      if (packageType === 'none' || packageType === 'commonjs') {
-        return 'commonjs';
-      } // Else packageType === 'module'
+    if (packageType === 'module') {
       return getFormatOfExtensionlessFile(url);
-    } // Else defaultType === 'module'
-    if (underNodeModules(url)) { // Exception for package scopes under `node_modules`
-      return packageType === 'module' ? getFormatOfExtensionlessFile(url) : 'commonjs';
     }
-    if (packageType === 'none' || packageType === 'module') {
-      return getFormatOfExtensionlessFile(url);
-    } // Else packageType === 'commonjs'
-    return 'commonjs';
+    if (packageType !== 'none') {
+      return packageType; // 'commonjs' or future package types
+    }
+
+    // The controlling `package.json` file has no `type` field.
+    switch (getOptionValue('--experimental-default-type')) {
+      case 'module': { // The user explicitly passed `--experimental-default-type=module`.
+        return underNodeModules(url) ? 'commonjs' : getFormatOfExtensionlessFile(url);
+      }
+      case 'commonjs': { // The user explicitly passed `--experimental-default-type=commonjs`.
+        return 'commonjs';
+      }
+      default: { // The user did not pass `--experimental-default-type`.
+        if (!source) {
+          return null;
+        }
+        const format = getFormatOfExtensionlessFile(url);
+        if (format === 'wasm') {
+          return format;
+        }
+        return detectModuleFormat(source, url);
+      }
+    }
   }
 
   const format = extensionFormatMap[ext];
   if (format) { return format; }
 
-  if (experimentalSpecifierResolution !== 'node') {
-    // Explicit undefined return indicates load hook should rerun format check
-    if (ignoreErrors) { return undefined; }
-    const filepath = fileURLToPath(url);
-    throw new ERR_UNKNOWN_FILE_EXTENSION(ext, filepath);
-  }
-
-  return getLegacyExtensionFormat(ext) ?? null;
+  // Explicit undefined return indicates load hook should rerun format check
+  if (ignoreErrors) { return undefined; }
+  const filepath = fileURLToPath(url);
+  throw new ERR_UNKNOWN_FILE_EXTENSION(ext, filepath);
 }
 
 /**
