@@ -75,10 +75,11 @@ function initializeDefaultConditions() {
   const userConditions = getOptionValue('--conditions');
   const noAddons = getOptionValue('--no-addons');
   const addonConditions = noAddons ? [] : ['node-addons'];
-
+  const moduleConditions = getOptionValue('--experimental-require-module') ? ['module-sync'] : [];
   defaultConditions = ObjectFreeze([
     'node',
     'import',
+    ...moduleConditions,
     ...addonConditions,
     ...userConditions,
   ]);
@@ -343,7 +344,7 @@ function compileSourceTextModule(url, source, cascadedLoader) {
   }
   // Cache the source map for the module if present.
   if (wrap.sourceMapURL) {
-    maybeCacheSourceMap(url, source, null, false, undefined, wrap.sourceMapURL);
+    maybeCacheSourceMap(url, source, wrap, false, undefined, wrap.sourceMapURL);
   }
   return wrap;
 }
