@@ -2,11 +2,12 @@
 
 const {
   ArrayIsArray,
+  ArrayPrototypeFilter,
+  ArrayPrototypePush,
   BigIntPrototypeValueOf,
   BooleanPrototypeValueOf,
   DatePrototypeGetTime,
   Error,
-  Map,
   NumberIsNaN,
   NumberPrototypeValueOf,
   ObjectGetOwnPropertySymbols,
@@ -16,10 +17,11 @@ const {
   ObjectPrototypeHasOwnProperty,
   ObjectPrototypePropertyIsEnumerable,
   ObjectPrototypeToString,
-  Set,
+  SafeSet,
   StringPrototypeValueOf,
   SymbolPrototypeValueOf,
-  SymbolToStringTag,
+  TypedArrayPrototypeGetSymbolToStringTag,
+  TypedArrayPrototypeGetByteLength,
   Uint8Array,
 } = primordials;
 
@@ -42,22 +44,15 @@ const {
   isSymbolObject,
   isFloat32Array,
   isFloat64Array,
-  isUint8Array,
-  isUint8ClampedArray,
-  isUint16Array,
-  isUint32Array,
-  isInt8Array,
-  isInt16Array,
-  isInt32Array,
-  isBigInt64Array,
-  isBigUint64Array
+  isKeyObject,
+  isCryptoKey,
 } = types;
 const {
-  getOwnNonIndexProperties,
-  propertyFilter: {
+  constants: {
     ONLY_ENUMERABLE,
-    SKIP_SYMBOLS
-  }
+    SKIP_SYMBOLS,
+  },
+  getOwnNonIndexProperties,
 } = internalBinding('util');
 
 const kStrict = true;
@@ -68,16 +63,21 @@ const kIsArray = 1;
 const kIsSet = 2;
 const kIsMap = 3;
 
+let kKeyObject;
+
 // Check if they have the same source and flags
 function areSimilarRegExps(a, b) {
-  return a.source === b.source && a.flags === b.flags;
+  return a.source === b.source &&
+         a.flags === b.flags &&
+         a.lastIndex === b.lastIndex;
 }
 
 function areSimilarFloatArrays(a, b) {
   if (a.byteLength !== b.byteLength) {
     return false;
   }
-  for (let offset = 0; offset < a.byteLength; offset++) {
+  const len = TypedArrayPrototypeGetByteLength(a);
+  for (let offset = 0; offset < len; offset++) {
     if (a[offset] !== b[offset]) {
       return false;
     }
@@ -124,37 +124,6 @@ function isEqualBoxedPrimitive(val1, val2) {
   assert.fail(`Unknown boxed type ${val1}`);
 }
 
-function isIdenticalTypedArrayType(a, b) {
-  // Fast path to reduce type checks in the common case.
-  const check = types[`is${a[SymbolToStringTag]}`];
-  if (check !== undefined && check(a)) {
-    return check(b);
-  }
-  // Manipulated Symbol.toStringTag.
-  for (const check of [
-    isUint16Array,
-    isUint32Array,
-    isInt8Array,
-    isInt16Array,
-    isInt32Array,
-    isFloat32Array,
-    isFloat64Array,
-    isBigInt64Array,
-    isBigUint64Array,
-    isUint8ClampedArray,
-    isUint8Array
-  ]) {
-    if (check(a)) {
-      return check(b);
-    }
-  }
-  /* c8 ignore next 4 */
-  assert.fail(
-    `Unknown TypedArray type checking ${a[SymbolToStringTag]} ${a}\n` +
-    `and ${b[SymbolToStringTag]} ${b}`
-  );
-}
-
 // Notes: Type tags are historical [[Class]] properties that can be set by
 // FunctionTemplate::SetClassName() in C++ or Symbol.toStringTag in JS
 // and retrieved using Object.prototype.toString.call(obj) in JS
@@ -170,28 +139,26 @@ function isIdenticalTypedArrayType(a, b) {
 function innerDeepEqual(val1, val2, strict, memos) {
   // All identical values are equivalent, as determined by ===.
   if (val1 === val2) {
-    if (val1 !== 0)
-      return true;
-    return strict ? ObjectIs(val1, val2) : true;
+    return val1 !== 0 || ObjectIs(val1, val2) || !strict;
   }
 
   // Check more closely if val1 and val2 are equal.
   if (strict) {
-    if (typeof val1 !== 'object') {
-      return typeof val1 === 'number' && NumberIsNaN(val1) &&
-        NumberIsNaN(val2);
+    if (typeof val1 === 'number') {
+      return NumberIsNaN(val1) && NumberIsNaN(val2);
     }
-    if (typeof val2 !== 'object' || val1 === null || val2 === null) {
-      return false;
-    }
-    if (ObjectGetPrototypeOf(val1) !== ObjectGetPrototypeOf(val2)) {
+    if (typeof val2 !== 'object' ||
+        typeof val1 !== 'object' ||
+        val1 === null ||
+        val2 === null ||
+        ObjectGetPrototypeOf(val1) !== ObjectGetPrototypeOf(val2)) {
       return false;
     }
   } else {
     if (val1 === null || typeof val1 !== 'object') {
       if (val2 === null || typeof val2 !== 'object') {
         // eslint-disable-next-line eqeqeq
-        return val1 == val2;
+        return val1 == val2 || (NumberIsNaN(val1) && NumberIsNaN(val2));
       }
       return false;
     }
@@ -229,17 +196,11 @@ function innerDeepEqual(val1, val2, strict, memos) {
     if (!isRegExp(val2) || !areSimilarRegExps(val1, val2)) {
       return false;
     }
-  } else if (isNativeError(val1) || val1 instanceof Error) {
-    // Do not compare the stack as it might differ even though the error itself
-    // is otherwise identical.
-    if ((!isNativeError(val2) && !(val2 instanceof Error)) ||
-        val1.message !== val2.message ||
-        val1.name !== val2.name) {
+  } else if (isArrayBufferView(val1)) {
+    if (TypedArrayPrototypeGetSymbolToStringTag(val1) !==
+        TypedArrayPrototypeGetSymbolToStringTag(val2)) {
       return false;
     }
-  } else if (isArrayBufferView(val1)) {
-    if (!isIdenticalTypedArrayType(val1, val2))
-      return false;
     if (!strict && (isFloat32Array(val1) || isFloat64Array(val1))) {
       if (!areSimilarFloatArrays(val1, val2)) {
         return false;
@@ -271,6 +232,28 @@ function innerDeepEqual(val1, val2, strict, memos) {
     if (!isAnyArrayBuffer(val2) || !areEqualArrayBuffers(val1, val2)) {
       return false;
     }
+  } else if (isNativeError(val1) || val1 instanceof Error) {
+    // Do not compare the stack as it might differ even though the error itself
+    // is otherwise identical.
+    if (!isNativeError(val2) && !(val2 instanceof Error)) {
+      return false;
+    }
+
+    const message1Enumerable = ObjectPrototypePropertyIsEnumerable(val1, 'message');
+    const name1Enumerable = ObjectPrototypePropertyIsEnumerable(val1, 'name');
+    const cause1Enumerable = ObjectPrototypePropertyIsEnumerable(val1, 'cause');
+    const errors1Enumerable = ObjectPrototypePropertyIsEnumerable(val1, 'errors');
+
+    if ((message1Enumerable !== ObjectPrototypePropertyIsEnumerable(val2, 'message') ||
+          (!message1Enumerable && val1.message !== val2.message)) ||
+        (name1Enumerable !== ObjectPrototypePropertyIsEnumerable(val2, 'name') ||
+          (!name1Enumerable && val1.name !== val2.name)) ||
+        (cause1Enumerable !== ObjectPrototypePropertyIsEnumerable(val2, 'cause') ||
+          (!cause1Enumerable && !innerDeepEqual(val1.cause, val2.cause, strict, memos))) ||
+        (errors1Enumerable !== ObjectPrototypePropertyIsEnumerable(val2, 'errors') ||
+         (!errors1Enumerable && !innerDeepEqual(val1.errors, val2.errors, strict, memos)))) {
+      return false;
+    }
   } else if (isBoxedPrimitive(val1)) {
     if (!isEqualBoxedPrimitive(val1, val2)) {
       return false;
@@ -286,12 +269,29 @@ function innerDeepEqual(val1, val2, strict, memos) {
              isNativeError(val2) ||
              val2 instanceof Error) {
     return false;
+  } else if (isKeyObject(val1)) {
+    if (!isKeyObject(val2) || !val1.equals(val2)) {
+      return false;
+    }
+  } else if (isCryptoKey(val1)) {
+    kKeyObject ??= require('internal/crypto/util').kKeyObject;
+    if (!isCryptoKey(val2) ||
+      val1.extractable !== val2.extractable ||
+      !innerDeepEqual(val1.algorithm, val2.algorithm, strict, memos) ||
+      !innerDeepEqual(val1.usages, val2.usages, strict, memos) ||
+      !innerDeepEqual(val1[kKeyObject], val2[kKeyObject], strict, memos)
+    ) {
+      return false;
+    }
   }
   return keyCheck(val1, val2, strict, memos, kNoIterator);
 }
 
 function getEnumerables(val, keys) {
-  return keys.filter((k) => ObjectPrototypePropertyIsEnumerable(val, k));
+  return ArrayPrototypeFilter(
+    keys,
+    (k) => ObjectPrototypePropertyIsEnumerable(val, k),
+  );
 }
 
 function keyCheck(val1, val2, strict, memos, iterationType, aKeys) {
@@ -302,50 +302,53 @@ function keyCheck(val1, val2, strict, memos, iterationType, aKeys) {
   // c) Equivalent values for every corresponding key/index
   // d) For Sets and Maps, equal contents
   // Note: this accounts for both named and indexed properties on Arrays.
-  if (arguments.length === 5) {
-    aKeys = ObjectKeys(val1);
-    const bKeys = ObjectKeys(val2);
+  const isArrayLikeObject = aKeys !== undefined;
 
-    // The pair must have the same number of owned properties.
-    if (aKeys.length !== bKeys.length) {
-      return false;
-    }
+  if (aKeys === undefined) {
+    aKeys = ObjectKeys(val1);
   }
 
   // Cheap key test
-  let i = 0;
-  for (; i < aKeys.length; i++) {
-    if (!ObjectPrototypeHasOwnProperty(val2, aKeys[i])) {
-      return false;
+  if (aKeys.length > 0) {
+    for (const key of aKeys) {
+      if (!ObjectPrototypePropertyIsEnumerable(val2, key)) {
+        return false;
+      }
     }
   }
 
-  if (strict && arguments.length === 5) {
-    const symbolKeysA = ObjectGetOwnPropertySymbols(val1);
-    if (symbolKeysA.length !== 0) {
-      let count = 0;
-      for (i = 0; i < symbolKeysA.length; i++) {
-        const key = symbolKeysA[i];
-        if (ObjectPrototypePropertyIsEnumerable(val1, key)) {
-          if (!ObjectPrototypePropertyIsEnumerable(val2, key)) {
+  if (!isArrayLikeObject) {
+    // The pair must have the same number of owned properties.
+    if (aKeys.length !== ObjectKeys(val2).length) {
+      return false;
+    }
+
+    if (strict) {
+      const symbolKeysA = ObjectGetOwnPropertySymbols(val1);
+      if (symbolKeysA.length !== 0) {
+        let count = 0;
+        for (const key of symbolKeysA) {
+          if (ObjectPrototypePropertyIsEnumerable(val1, key)) {
+            if (!ObjectPrototypePropertyIsEnumerable(val2, key)) {
+              return false;
+            }
+            ArrayPrototypePush(aKeys, key);
+            count++;
+          } else if (ObjectPrototypePropertyIsEnumerable(val2, key)) {
             return false;
           }
-          aKeys.push(key);
-          count++;
-        } else if (ObjectPrototypePropertyIsEnumerable(val2, key)) {
+        }
+        const symbolKeysB = ObjectGetOwnPropertySymbols(val2);
+        if (symbolKeysA.length !== symbolKeysB.length &&
+            getEnumerables(val2, symbolKeysB).length !== count) {
+          return false;
+        }
+      } else {
+        const symbolKeysB = ObjectGetOwnPropertySymbols(val2);
+        if (symbolKeysB.length !== 0 &&
+            getEnumerables(val2, symbolKeysB).length !== 0) {
           return false;
         }
-      }
-      const symbolKeysB = ObjectGetOwnPropertySymbols(val2);
-      if (symbolKeysA.length !== symbolKeysB.length &&
-          getEnumerables(val2, symbolKeysB).length !== count) {
-        return false;
-      }
-    } else {
-      const symbolKeysB = ObjectGetOwnPropertySymbols(val2);
-      if (symbolKeysB.length !== 0 &&
-          getEnumerables(val2, symbolKeysB).length !== 0) {
-        return false;
       }
     }
   }
@@ -360,45 +363,68 @@ function keyCheck(val1, val2, strict, memos, iterationType, aKeys) {
   // Use memos to handle cycles.
   if (memos === undefined) {
     memos = {
-      val1: new Map(),
-      val2: new Map(),
-      position: 0
+      set: undefined,
+      a: val1,
+      b: val2,
+      c: undefined,
+      d: undefined,
+      deep: false,
+      deleteFailures: false,
     };
-  } else {
-    // We prevent up to two map.has(x) calls by directly retrieving the value
-    // and checking for undefined. The map can only contain numbers, so it is
-    // safe to check for undefined only.
-    const val2MemoA = memos.val1.get(val1);
-    if (val2MemoA !== undefined) {
-      const val2MemoB = memos.val2.get(val2);
-      if (val2MemoB !== undefined) {
-        return val2MemoA === val2MemoB;
-      }
-    }
-    memos.position++;
+    return objEquiv(val1, val2, strict, aKeys, memos, iterationType);
   }
 
-  memos.val1.set(val1, memos.position);
-  memos.val2.set(val2, memos.position);
+  if (memos.set === undefined) {
+    if (memos.deep === false) {
+      if (memos.a === val1) {
+        if (memos.b === val2) return true;
+      }
+      memos.c = val1;
+      memos.d = val2;
+      memos.deep = true;
+      const result = objEquiv(val1, val2, strict, aKeys, memos, iterationType);
+      memos.deep = false;
+      return result;
+    }
+    memos.set = new SafeSet();
+    memos.set.add(memos.a);
+    memos.set.add(memos.b);
+    memos.set.add(memos.c);
+    memos.set.add(memos.d);
+  }
+
+  const { set } = memos;
+
+  const originalSize = set.size;
+  set.add(val1);
+  set.add(val2);
+  if (originalSize === set.size) {
+    return true;
+  }
 
   const areEq = objEquiv(val1, val2, strict, aKeys, memos, iterationType);
 
-  memos.val1.delete(val1);
-  memos.val2.delete(val2);
+  if (areEq || memos.deleteFailures) {
+    set.delete(val1);
+    set.delete(val2);
+  }
 
   return areEq;
 }
 
-function setHasEqualElement(set, val1, strict, memo) {
-  // Go looking.
-  for (const val2 of set) {
+function setHasEqualElement(set, val2, strict, memo) {
+  const { deleteFailures } = memo;
+  memo.deleteFailures = true;
+  for (const val1 of set) {
     if (innerDeepEqual(val1, val2, strict, memo)) {
       // Remove the matching element to make sure we do not check that again.
-      set.delete(val2);
+      set.delete(val1);
+      memo.deleteFailures = deleteFailures;
       return true;
     }
   }
 
+  memo.deleteFailures = deleteFailures;
   return false;
 }
 
@@ -453,41 +479,32 @@ function setEquiv(a, b, strict, memo) {
   // pairwise.
   let set = null;
   for (const val of a) {
-    // Note: Checking for the objects first improves the performance for object
-    // heavy sets but it is a minor slow down for primitives. As they are fast
-    // to check this improves the worst case scenario instead.
-    if (typeof val === 'object' && val !== null) {
-      if (set === null) {
-        set = new Set();
-      }
-      // If the specified value doesn't exist in the second set its an not null
-      // object (or non strict only: a not matching primitive) we'll need to go
-      // hunting for something thats deep-(strict-)equal to it. To make this
-      // O(n log n) complexity we have to copy these values in a new set first.
-      set.add(val);
-    } else if (!b.has(val)) {
-      if (strict)
-        return false;
-
-      // Fast path to detect missing string, symbol, undefined and null values.
-      if (!setMightHaveLoosePrim(a, b, val)) {
+    if (!b.has(val)) {
+      if ((typeof val !== 'object' || val === null) &&
+          (strict || !setMightHaveLoosePrim(a, b, val))) {
         return false;
       }
 
       if (set === null) {
-        set = new Set();
+        if (b.size === 1) {
+          return innerDeepEqual(val, b.values().next().value, strict, memo);
+        }
+        set = new SafeSet();
       }
+      // If the specified value doesn't exist in the second set it's a object
+      // (or in loose mode: a non-matching primitive). Find the
+      // deep-(strict-)equal element in a set copy to reduce duplicate checks.
       set.add(val);
     }
   }
 
   if (set !== null) {
     for (const val of b) {
-      // We have to check if a primitive value is already
-      // matching and only if it's not, go hunting for it.
+      // Primitive values have already been handled above.
       if (typeof val === 'object' && val !== null) {
-        if (!setHasEqualElement(set, val, strict, memo))
+        if (!a.has(val) && !setHasEqualElement(set, val, strict, memo)) {
           return false;
+        }
       } else if (!strict &&
                  !a.has(val) &&
                  !setHasEqualElement(set, val, strict, memo)) {
@@ -500,28 +517,37 @@ function setEquiv(a, b, strict, memo) {
   return true;
 }
 
-function mapHasEqualEntry(set, map, key1, item1, strict, memo) {
+function mapHasEqualEntry(set, map, key2, item2, strict, memo) {
   // To be able to handle cases like:
   //   Map([[{}, 'a'], [{}, 'b']]) vs Map([[{}, 'b'], [{}, 'a']])
   // ... we need to consider *all* matching keys, not just the first we find.
-  for (const key2 of set) {
+  const { deleteFailures } = memo;
+  memo.deleteFailures = true;
+  for (const key1 of set) {
     if (innerDeepEqual(key1, key2, strict, memo) &&
-      innerDeepEqual(item1, map.get(key2), strict, memo)) {
-      set.delete(key2);
+      innerDeepEqual(map.get(key1), item2, strict, memo)) {
+      set.delete(key1);
+      memo.deleteFailures = deleteFailures;
       return true;
     }
   }
 
+  memo.deleteFailures = deleteFailures;
   return false;
 }
 
 function mapEquiv(a, b, strict, memo) {
   let set = null;
 
-  for (const [key, item1] of a) {
+  for (const { 0: key, 1: item1 } of a) {
     if (typeof key === 'object' && key !== null) {
       if (set === null) {
-        set = new Set();
+        if (b.size === 1) {
+          const { 0: key2, 1: item2 } = b.entries().next().value;
+          return innerDeepEqual(key, key2, strict, memo) &&
+                 innerDeepEqual(item1, item2, strict, memo);
+        }
+        set = new SafeSet();
       }
       set.add(key);
     } else {
@@ -537,7 +563,7 @@ function mapEquiv(a, b, strict, memo) {
         if (!mapMightHaveLoosePrim(a, b, key, item1, memo))
           return false;
         if (set === null) {
-          set = new Set();
+          set = new SafeSet();
         }
         set.add(key);
       }
@@ -545,14 +571,14 @@ function mapEquiv(a, b, strict, memo) {
   }
 
   if (set !== null) {
-    for (const [key, item] of b) {
+    for (const { 0: key, 1: item } of b) {
       if (typeof key === 'object' && key !== null) {
         if (!mapHasEqualEntry(set, a, key, item, strict, memo))
           return false;
       } else if (!strict &&
                  (!a.has(key) ||
-                   !innerDeepEqual(a.get(key), item, false, memo)) &&
-                 !mapHasEqualEntry(set, a, key, item, false, memo)) {
+                   !innerDeepEqual(a.get(key), item, strict, memo)) &&
+                 !mapHasEqualEntry(set, a, key, item, strict, memo)) {
         return false;
       }
     }
@@ -563,20 +589,17 @@ function mapEquiv(a, b, strict, memo) {
 }
 
 function objEquiv(a, b, strict, keys, memos, iterationType) {
-  // Sets and maps don't have their entries accessible via normal object
-  // properties.
-  let i = 0;
+  // The pair must have equivalent values for every corresponding key.
+  if (keys.length > 0) {
+    for (const key of keys) {
+      if (!innerDeepEqual(a[key], b[key], strict, memos)) {
+        return false;
+      }
+    }
+  }
 
-  if (iterationType === kIsSet) {
-    if (!setEquiv(a, b, strict, memos)) {
-      return false;
-    }
-  } else if (iterationType === kIsMap) {
-    if (!mapEquiv(a, b, strict, memos)) {
-      return false;
-    }
-  } else if (iterationType === kIsArray) {
-    for (; i < a.length; i++) {
+  if (iterationType === kIsArray) {
+    for (let i = 0; i < a.length; i++) {
       if (ObjectPrototypeHasOwnProperty(a, i)) {
         if (!ObjectPrototypeHasOwnProperty(b, i) ||
             !innerDeepEqual(a[i], b[i], strict, memos)) {
@@ -594,22 +617,19 @@ function objEquiv(a, b, strict, keys, memos, iterationType) {
             return false;
           }
         }
-        if (keysA.length !== ObjectKeys(b).length) {
-          return false;
-        }
-        return true;
+        return keysA.length === ObjectKeys(b).length;
       }
     }
-  }
-
-  // The pair must have equivalent values for every corresponding key.
-  // Possibly expensive deep test:
-  for (i = 0; i < keys.length; i++) {
-    const key = keys[i];
-    if (!innerDeepEqual(a[key], b[key], strict, memos)) {
+  } else if (iterationType === kIsSet) {
+    if (!setEquiv(a, b, strict, memos)) {
+      return false;
+    }
+  } else if (iterationType === kIsMap) {
+    if (!mapEquiv(a, b, strict, memos)) {
       return false;
     }
   }
+
   return true;
 }
 
@@ -623,5 +643,5 @@ function isDeepStrictEqual(val1, val2) {
 
 module.exports = {
   isDeepEqual,
-  isDeepStrictEqual
+  isDeepStrictEqual,
 };
