@@ -230,11 +230,12 @@ function createAsyncFromSyncIterator(syncIteratorRecord) {
   return { iterator: asyncIterator, nextMethod, done: false };
 }
 
+// Refs: https://tc39.es/ecma262/#sec-getiterator
 function getIterator(obj, kind = 'sync', method) {
   if (method === undefined) {
     if (kind === 'async') {
       method = obj[SymbolAsyncIterator];
-      if (method === undefined) {
+      if (method == null) {
         const syncMethod = obj[SymbolIterator];
 
         if (syncMethod === undefined) {
