@@ -6,6 +6,7 @@ let dot;
 let junit;
 let spec;
 let tap;
+let lcov;
 
 ObjectDefineProperties(module.exports, {
   __proto__: null,
@@ -45,4 +46,13 @@ ObjectDefineProperties(module.exports, {
       return tap;
     },
   },
+  lcov: {
+    __proto__: null,
+    configurable: true,
+    enumerable: true,
+    get() {
+      lcov ??= require('internal/test_runner/reporter/lcov');
+      return ReflectConstruct(lcov, arguments);
+    },
+  },
 });
