@@ -4,15 +4,16 @@
 const common = require('../common');
 if (!common.hasCrypto)
   common.skip('missing crypto');
-if (process.config.variables.asan)
-  common.skip('ASAN messes with memory measurements');
+if (common.isASan)
+  common.skip('ASan messes with memory measurements');
 
 const assert = require('assert');
 const crypto = require('crypto');
 
-const before = process.memoryUsage().rss;
+const before = process.memoryUsage.rss();
 {
-  const dh = crypto.createDiffieHellman(common.hasFipsCrypto ? 1024 : 256);
+  const size = common.hasFipsCrypto || common.hasOpenSSL3 ? 1024 : 256;
+  const dh = crypto.createDiffieHellman(size);
   const publicKey = dh.generateKeys();
   const privateKey = dh.getPrivateKey();
   for (let i = 0; i < 5e4; i += 1) {
@@ -21,7 +22,7 @@ const before = process.memoryUsage().rss;
   }
 }
 global.gc();
-const after = process.memoryUsage().rss;
+const after = process.memoryUsage.rss();
 
 // RSS should stay the same, ceteris paribus, but allow for
 // some slop because V8 mallocs memory during execution.
