@@ -7,14 +7,13 @@ const fixtures = require('../common/fixtures');
 const assert = require('assert');
 const http2 = require('http2');
 const fs = require('fs');
-const path = require('path');
 
 // Piping should work as expected with createWriteStream
 
 const tmpdir = require('../common/tmpdir');
 tmpdir.refresh();
 const loc = fixtures.path('person-large.jpg');
-const fn = path.join(tmpdir.path, 'http2-url-tests.js');
+const fn = tmpdir.resolve('http2-url-tests.js');
 
 const server = http2.createServer();
 
