@@ -3,12 +3,13 @@
 const common = require('../common');
 const ArrayStream = require('../common/arraystream');
 const assert = require('assert');
+const events = require('events');
 const { stripVTControlCharacters } = require('internal/util/inspect');
 const repl = require('repl');
 
 common.skipIfInspectorDisabled();
 
-// Flags: --expose-internals --experimental-repl-await
+// Flags: --expose-internals
 
 const PROMPT = 'await repl > ';
 
@@ -27,31 +28,21 @@ class REPLStream extends ArrayStream {
     if (chunkLines.length > 1) {
       this.lines.push(...chunkLines.slice(1));
     }
-    this.emit('line');
+    this.emit('line', this.lines[this.lines.length - 1]);
     if (callback) callback();
     return true;
   }
 
-  wait() {
+  async wait() {
     if (this.waitingForResponse) {
       throw new Error('Currently waiting for response to another command');
     }
     this.lines = [''];
-    return new Promise((resolve, reject) => {
-      const onError = (err) => {
-        this.removeListener('line', onLine);
-        reject(err);
-      };
-      const onLine = () => {
-        if (this.lines[this.lines.length - 1].includes(PROMPT)) {
-          this.removeListener('error', onError);
-          this.removeListener('line', onLine);
-          resolve(this.lines);
-        }
-      };
-      this.once('error', onError);
-      this.on('line', onLine);
-    });
+    for await (const [line] of events.on(this, 'line')) {
+      if (line.includes(PROMPT)) {
+        return this.lines;
+      }
+    }
   }
 }
 
@@ -81,7 +72,7 @@ async function ordinaryTests() {
   // https://cs.chromium.org/chromium/src/third_party/WebKit/LayoutTests/http/tests/devtools/console/console-top-level-await.js?rcl=5d0ea979f0ba87655b7ef0e03b58fa3c04986ba6
   putIn.run([
     'function foo(x) { return x; }',
-    'function koo() { return Promise.resolve(4); }'
+    'function koo() { return Promise.resolve(4); }',
   ]);
   const testCases = [
     ['await Promise.resolve(0)', '0'],
@@ -139,9 +130,53 @@ async function ordinaryTests() {
        '1',
        '2',
        '3',
-       'undefined'
-     ]
-    ]
+       'undefined',
+     ],
+    ],
+    ['await Promise..resolve()',
+     [
+       'await Promise..resolve()\r',
+       'Uncaught SyntaxError: ',
+       'await Promise..resolve()',
+       '              ^',
+       '',
+       'Unexpected token \'.\'',
+     ],
+    ],
+    ['for (const x of [1,2,3]) {\nawait x\n}', [
+      'for (const x of [1,2,3]) {\r',
+      '... await x\r',
+      '... }\r',
+      'undefined',
+    ]],
+    ['for (const x of [1,2,3]) {\nawait x;\n}', [
+      'for (const x of [1,2,3]) {\r',
+      '... await x;\r',
+      '... }\r',
+      'undefined',
+    ]],
+    ['for await (const x of [1,2,3]) {\nconsole.log(x)\n}', [
+      'for await (const x of [1,2,3]) {\r',
+      '... console.log(x)\r',
+      '... }\r',
+      '1',
+      '2',
+      '3',
+      'undefined',
+    ]],
+    ['for await (const x of [1,2,3]) {\nconsole.log(x);\n}', [
+      'for await (const x of [1,2,3]) {\r',
+      '... console.log(x);\r',
+      '... }\r',
+      '1',
+      '2',
+      '3',
+      'undefined',
+    ]],
+    // Regression test for https://github.com/nodejs/node/issues/43777.
+    ['await Promise.resolve(123), Promise.resolve(456)', 'Promise {', { line: 0 }],
+    ['await Promise.resolve(123), await Promise.resolve(456)', '456'],
+    ['await (Promise.resolve(123), Promise.resolve(456))', '456'],
   ];
 
   for (const [input, expected = [`${input}\r`], options = {}] of testCases) {
@@ -164,24 +199,20 @@ async function ordinaryTests() {
 }
 
 async function ctrlCTest() {
-  putIn.run([
-    `const timeout = (msecs) => new Promise((resolve) => {
-       setTimeout(resolve, msecs).unref();
-     });`
-  ]);
-
   console.log('Testing Ctrl+C');
-  assert.deepStrictEqual(await runAndWait([
-    'await timeout(100000)',
-    { ctrl: true, name: 'c' }
-  ]), [
-    'await timeout(100000)\r',
+  const output = await runAndWait([
+    'await new Promise(() => {})',
+    { ctrl: true, name: 'c' },
+  ]);
+  assert.deepStrictEqual(output.slice(0, 3), [
+    'await new Promise(() => {})\r',
     'Uncaught:',
     '[Error [ERR_SCRIPT_EXECUTION_INTERRUPTED]: ' +
       'Script execution was interrupted by `SIGINT`] {',
-    "  code: 'ERR_SCRIPT_EXECUTION_INTERRUPTED'",
+  ]);
+  assert.deepStrictEqual(output.slice(-2), [
     '}',
-    PROMPT
+    PROMPT,
   ]);
 }
 
@@ -190,4 +221,4 @@ async function main() {
   await ctrlCTest();
 }
 
-main();
+main().then(common.mustCall());
