@@ -8,11 +8,10 @@ const assert = require('assert');
 const { spawnSync } = require('child_process');
 const tmpdir = require('../common/tmpdir');
 const fixtures = require('../common/fixtures');
-const path = require('path');
 const fs = require('fs');
 
 tmpdir.refresh();
-const blobPath = path.join(tmpdir.path, 'snapshot.blob');
+const blobPath = tmpdir.resolve('snapshot.blob');
 const file = fixtures.path('snapshot', 'mutate-fs.js');
 const checkFile = fixtures.path('snapshot', 'check-mutate-fs.js');
 
