@@ -12,7 +12,6 @@ try {
 
 const assert = require('assert');
 const cp = require('child_process');
-const path = require('path');
 const fs = require('fs');
 const tmpdir = require('../common/tmpdir');
 const {
@@ -32,7 +31,7 @@ const isChild = process.argv[2] === 'child';
 const enabledCategories = getEnabledCategoriesFromCommandLine();
 
 assert.strictEqual(getEnabledCategories(), enabledCategories);
-[1, 'foo', true, false, null, undefined].forEach((i) => {
+for (const i of [1, 'foo', true, false, null, undefined]) {
   assert.throws(() => createTracing(i), {
     code: 'ERR_INVALID_ARG_TYPE',
     name: 'TypeError'
@@ -41,7 +40,7 @@ assert.strictEqual(getEnabledCategories(), enabledCategories);
     code: 'ERR_INVALID_ARG_TYPE',
     name: 'TypeError'
   });
-});
+}
 
 assert.throws(
   () => createTracing({ categories: [] }),
@@ -147,7 +146,7 @@ function testApiInChildProcess(execArgs, cb) {
                        });
 
   proc.once('exit', common.mustCall(() => {
-    const file = path.join(tmpdir.path, 'node_trace.1.log');
+    const file = tmpdir.resolve('node_trace.1.log');
 
     assert(fs.existsSync(file));
     fs.readFile(file, common.mustSucceed((data) => {
@@ -157,8 +156,7 @@ function testApiInChildProcess(execArgs, cb) {
       assert.strictEqual(
         traces.length,
         expectedBegins.length + expectedEnds.length);
-
-      traces.forEach((trace) => {
+      for (const trace of traces) {
         assert.strictEqual(trace.pid, proc.pid);
         switch (trace.ph) {
           case 'b': {
@@ -176,7 +174,7 @@ function testApiInChildProcess(execArgs, cb) {
           default:
             assert.fail('Unexpected trace event phase');
         }
-      });
+      }
       process.chdir(parentDir);
       cb && process.nextTick(cb);
     }));
