@@ -5,7 +5,6 @@
 const common = require('../common');
 const assert = require('assert');
 const zlib = require('zlib');
-const path = require('path');
 const fixtures = require('../common/fixtures');
 
 const tmpdir = require('../common/tmpdir');
@@ -17,7 +16,7 @@ const fs = require('fs');
 
 const fixture = fixtures.path('person.jpg.br');
 const unzippedFixture = fixtures.path('person.jpg');
-const outputFile = path.resolve(tmpdir.path, 'person.jpg');
+const outputFile = tmpdir.resolve('person.jpg');
 const expect = fs.readFileSync(unzippedFixture);
 const inp = fs.createReadStream(fixture);
 const out = fs.createWriteStream(outputFile);
