@@ -1,5 +1,5 @@
 'use strict';
-require('../common');
+const common = require('../common');
 
 if (process.argv[2] === 'wasi-child') {
   const assert = require('assert');
@@ -8,6 +8,7 @@ if (process.argv[2] === 'wasi-child') {
 
   const { WASI } = require('wasi');
   const wasi = new WASI({
+    version: 'preview1',
     args: ['foo', '-bar', '--baz=value'],
   });
   const importObject = { wasi_snapshot_preview1: wasi.wasiImport };
@@ -22,7 +23,7 @@ if (process.argv[2] === 'wasi-child') {
     name: 'Error',
     code: 'ERR_WASI_NOT_STARTED',
     message: 'wasi.start() has not been called',
-  });
+  }).then(common.mustCall());
 } else {
   const assert = require('assert');
   const cp = require('child_process');
